/*
  RCTractor.h - Library for controlling RC Tractors.
  Created by Oisin O'Conchubhair, September 20, 2015.
  Released into the public domain.
*/

#include "Arduino.h"
#include "RCTractorControllerV2.h"
#include "RCTractorControllerV2MenuBackend.h"
#include "RCTractorControllerEncoder.h"
#include <RCTractorControllerAdafruit_GFX.h>    // Core graphics library
#include <RCTractorControllerAdafruit_TFTLCD.h> // Hardware-specific library
#include <SD.h>
#include <RCTractorControllerIRremote.h>
#include <SPI.h>
#include "RCTractorControllernRF24L01.h"
#include "RCTractorControllerRF24.h"
#include <RCTractorControllerV2TouchScreen.h>

// Single radio pipe address for the 2 nodes to communicate.
const uint64_t Controller_Send[5] = { 0xE8E8F0F0E1LL, 0xE8E8F0F0E2LL, 0xE8E8F0F0E3LL, 0xE8E8F0F0E4LL, 0xE8E8F0F0E5LL};
const uint64_t Controller_Listen[5] = { 0x3A3A3A3AD1LL, 0x3A3A3A3AD2LL, 0x3A3A3A3AD3LL, 0x3A3A3A3AD4LL, 0x3A3A3A3AD5LL };
static int Controller_Num=1;

// Set up nRF24L01 radio on SPI bus plus pins 9 & 10
RF24 radio(49, 53);
static bool Radio_Attached = 0;

/*******************************************************************/
/*                      TFT and SD Card Setup                      */
/*******************************************************************/

// The control pins for the LCD
#define LCD_CS A3 // Chip Select goes to Analog 3
#define LCD_CD A2 // Command/Data goes to Analog 2
#define LCD_WR A1 // LCD Write goes to Analog 1
#define LCD_RD A0 // LCD Read goes to Analog 0

// Assign human-readable names to some common 16-bit color values:
#define  BLACK   0x0000
#define BLUE    0x001F
#define RED     0xF800
#define GREEN   0x07E0
#define CYAN    0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE   0xFFFF
#define ORANGE  0xFB20

#define SD_CS 48     // Set the chip select line to whatever you use (10 doesnt conflict with the library)

#define BUFFPIXEL 20


// our TFT wiring
RCTractorControllerAdafruit_TFTLCD tft(LCD_CS, LCD_CD, LCD_WR, LCD_RD, A4);

RCTractorControllerV2 RC_Library_TC(21);

/*******************************************************************/
/*               Variables for Voltage Measurements                */
/*******************************************************************/

// We're going to average the voltage readings
static const int numReadings = 10;
static int Controller_V_Readings[numReadings];      // the readings from the analog input
static float Vehicle_V_Readings[numReadings];      // the readings from the analog input
static int Controller_V_ReadIndex = 0;              // the index of the current reading
static int Vehicle_V_ReadIndex = 0;              // the index of the current reading
static int Controller_V_Total = 0;                  // the running total
static float Vehicle_V_Total = 0;                  // the running total
static float Vehicle_V_Average = 0;                // the average
static int Controller_V_Measure_Period = 0;
static int Vehicle_V_Measure_Period = 0;
static float Denominator;
static float Controller_V_Average;
static float Old_Controller_V_Average;
static float Old_Vehicle_V_Average;
static float Vehicle_Volt;
static long Vehicle_V;

/*******************************************************************/
/*                          Encoder Setup                          */
/*******************************************************************/
RCTractorControllerEncoder Encoder1;
RCTractorControllerEncoder Encoder2;

static int Encoder1NewPosition = 127;
static int Encoder1OldPosition = 127;
static int Encoder2NewPosition = 127;
static int Encoder2OldPosition = 127;
static int Encoder1_Limit = 51;

static double old_fl_x = 0;
static double old_fl_y = 0;

/*******************************************************************/
/*                        Infrared Setup                           */
/*******************************************************************/

IRsend irsend(47);  // send on pin 10
static int IRHeli_Throttle = 0;
static int IRHeli_Pitch = 0;
static int IRHeli_Yaw = 0;
static int IRHeli_Trim = 31;
unsigned long Com_Method_Timer = 0;
byte IR32_BYTE1 = 0x80;

/*******************************************************************/
/*                        General Variables                        */
/*******************************************************************/

static int Loop_Counter = 0;
static int Current_Light_Symbol=0;
static int Flash_Timer = 0;
static int Vehicle_No = 255;
static int New_Vehicle_No = 255;
static uint8_t command[12];
static uint8_t old_command[12];
static int IR_Heli_Active = 0;
static int TV_Active = 0;
static int Game_Active = 0;
static int Tool_Active = 0;
static int Vehicle_Active = 0;
static int Vehicle_Gear = 3;
static int Dashx = 180; // Dashboard Origin Coord X
static int Dashy = 150; // Dashboard Origin Coord Y
static int Controllerx = 210; // Dashboard Origin Coord X
static int Controllery = 25; // Dashboard Origin Coord Y
static int Throttle_Position = 0;
static int Lost_Packet_Count = 0;
static int Reception_Meter = 0;
static int Battery_Meter = 0;
static int EN_Mode = 5;
static int Control32_Active = 0;
static int Humanoid_Active = 0;
const char* Current_Vehicle_Name;
static int Com_Method = 0;
static unsigned long Send_Timer;
uint16_t identifier;

/*******************************************************************/
/*                    Variables used by Games                      */
/*******************************************************************/

// Snake Variables
			// Set Game Resolution
			static const int scale = 4;
			static int xScreen = (tft.width()-100)/scale;
			static int yScreen = (tft.height()-160)/scale;

			// some global variables
			static const int snakeInitialLength = 12;
			static const int snakeMaxLength = 300; // max length of the snake
			static int snakeLength = snakeInitialLength; // length of the snake
			static int positions[2][snakeMaxLength]; // positions of the snake pieces
			static int beacon[2]; // poition of the beacon
			static int directionSnake = 0; // the direction where the snake moves
			static unsigned long lastTimeMoved; // last time in wich time snake moved
			static unsigned long score = (snakeLength - snakeInitialLength) * 9; //score
			static unsigned long lastScore = score; // score at the previous time
			const int scorePosition = 135; // position of the score in the screen

			// initialize the position of the snake
			static const int startX = 8; // position where the snake starts
			static const int startY = 5;

			static int slider = 0;

// Pong Variables
static int paddleX = 0;
static int paddleY = 0;
static int oldPaddleX, oldPaddleY;
static int ballDirectionX = 1;
static int ballDirectionY = 1;
static int ballSpeed = 10; // lower numbers are faster
static int ballX, ballY, oldBallX, oldBallY;

// Brick Variables

			///////////////////////////////////////////////////
			//variables for the position of the ball and paddle
			//in different ways when the paddle is hit in different spot1s
			static int paddle = 0;
			static int paddle2 = 0;
			static int paddle3 = 0;
			static int paddle4 = 0;
			static int paddle5 = 0;
			//top left corner of screen is 0,0.

			static int oldPaddle = 0;

			///////////////////////////////////////////////////////
			//Screen Characteristics
			static int screenWidth = tft.width(); //make a wild guess
			static int screenHeight = tft.height(); //make a wild guess
			///////////////////////////////////////////////////////////
			//Paddle Characteristics
			//The paddle is broken into 3 parts so the ball will move
			static int paddleSection = 4; //how big each section in the paddle is
			static boolean paddleMid = true; //middle paddle will change direction every other time
			static int paddleHeight = 5;
			static int paddleWidth = paddleSection * 5; //make samller to make game harder ;)
			///////////////////////////////////////////////////////////
			//Ball Characteristics
			static int ballWidth = 5;
			static int ballHeight = 5;
			///////////////////////////////////////////////////////////
			//Player info
			static int numberLives = 3; //increase if you suck ;)
			static boolean win = false; //Chekcs if you are a winner
			static int bricksBroken = 0;
			///////////////////////////////////////////////////////////
			//Brick Characteristics
			static int brickWidth = 20;
			static int brickHeight = 5;
			static int bricksPerRow = 6;
			static int gap = 5; //gap in between bricks
			static int brickNumber[18];
			static int brickRow[3]; //which row the brick is on. Check break dem bricks
			static boolean brickHit[18] = {false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false,false}; //initialize all to values to false
			/////////////////////////////////////////////////////////////////////
			//Mouse Cursor Characteristics
			static int sideLength = 5; //length of the side of the triangle
			static int mouseX = 0;
			static int mouseY = 0;
			static int oldMouseX = 0;
			static int oldMouseY = 0;
			static int mouseSelect = 0;
			/////////////////////////////////////////////////////////////////////

			static boolean start = false;

/*******************************************************************/
/*                      Menu Setup and Control                     */
/*******************************************************************/

static int Menu_Change = 0;
static int Menu_Level = 0;

	//beneath is list of menu items needed to build the menu
	MenuItem MainMenu1 = MenuItem("RC Farm");
		MenuItem Farm1 = MenuItem("Ford TW35");
		MenuItem Farm2 = MenuItem("John Deere 6920S");
		MenuItem Farm3 = MenuItem("John Deere 8360RT");
		MenuItem Farm4 = MenuItem("John Deere 9560R");
		MenuItem Farm5 = MenuItem("Massey 8680");
		MenuItem Farm6 = MenuItem("Fendt 936");
		MenuItem Farm7 = MenuItem("Claas Xerion 5000");
		MenuItem Farm8 = MenuItem("Claas Jaguar");
		MenuItem Farm9 = MenuItem("Ford 7810");
		MenuItem Farm10 = MenuItem("Tractor 10");

	MenuItem MainMenu2 = MenuItem("RC Haulage");
		MenuItem Haulage1 = MenuItem("Scania R620");
		MenuItem Haulage2 = MenuItem("MAN Dumper");
		MenuItem Haulage3 = MenuItem("Volvo A40D");
		MenuItem Haulage4 = MenuItem("Truck 4");
		MenuItem Haulage5 = MenuItem("Truck 5");
		MenuItem Haulage6 = MenuItem("Truck 6");
		MenuItem Haulage7 = MenuItem("Truck 7");
		MenuItem Haulage8 = MenuItem("Truck 8");
		MenuItem Haulage9 = MenuItem("Truck 9");
		MenuItem Haulage10 = MenuItem("Subaru Impreza");

	MenuItem MainMenu3 = MenuItem("RC Construction");
		MenuItem Construction1 = MenuItem("Hitachi Zaxis 210");
		MenuItem Construction2 = MenuItem("Komatsu PC400LC");
		MenuItem Construction3 = MenuItem("Liebherr A900C");
		MenuItem Construction4 = MenuItem("JCB 550");
		MenuItem Construction5 = MenuItem("JCB JS330");
		MenuItem Construction6 = MenuItem("Construction 6");
		MenuItem Construction7 = MenuItem("Construction 7");
		MenuItem Construction8 = MenuItem("Construction 8");
		MenuItem Construction9 = MenuItem("Construction 9");
		MenuItem Construction10 = MenuItem("Construction 10");

	MenuItem MainMenu4 = MenuItem("RC Trailers");
		MenuItem Trailer1 = MenuItem("Kane Silage Trailer");
		MenuItem Trailer2 = MenuItem("Trailer 2");
		MenuItem Trailer3 = MenuItem("Trailer 3");
		MenuItem Trailer4 = MenuItem("Trailer 4");
		MenuItem Trailer5 = MenuItem("Trailer 5");
		MenuItem Trailer6 = MenuItem("Trailer 6");
		MenuItem Trailer7 = MenuItem("Trailer 7");
		MenuItem Trailer8 = MenuItem("Trailer 8");
		MenuItem Trailer9 = MenuItem("Trailer 9");
		MenuItem Trailer10 = MenuItem("Trailer 10");

	MenuItem MainMenu5 = MenuItem("IR Control 32");
		MenuItem SikuControl1 = MenuItem("Channel 1");
		MenuItem SikuControl2 = MenuItem("Channel 2");
		MenuItem SikuControl3 = MenuItem("Channel 3");
		MenuItem SikuControl4 = MenuItem("Channel 4");

	MenuItem MainMenu6 = MenuItem("IR Helicopters");
		MenuItem IRHeli1 = MenuItem("Syma S107 Chan A");
		MenuItem IRHeli2 = MenuItem("Syma S107 Chan B");
		MenuItem IRHeli3 = MenuItem("Syma S107 Chan C");
		MenuItem IRHeli4 = MenuItem("IR Heli 4");
		MenuItem IRHeli5 = MenuItem("IR Heli 5");
		MenuItem IRHeli6 = MenuItem("IR Heli 6");
		MenuItem IRHeli7 = MenuItem("IR Heli 7");
		MenuItem IRHeli8 = MenuItem("IR Heli 8");
		MenuItem IRHeli9 = MenuItem("IR Heli 9");
		MenuItem IRHeli10 = MenuItem("IR Heli 10");

	MenuItem MainMenu7 = MenuItem("Drones/Robots");
		MenuItem Drone1 = MenuItem("Heli Drone");
		MenuItem Drone2 = MenuItem("Plane Drone");
		MenuItem Drone3 = MenuItem("Walking Robot");
		MenuItem Drone4 = MenuItem("Robot Car");
		MenuItem Drone5 = MenuItem("Drone 5");
		MenuItem Drone6 = MenuItem("Drone 6");
		MenuItem Drone7 = MenuItem("Drone 7");
		MenuItem Drone8 = MenuItem("Drone 8");
		MenuItem Drone9 = MenuItem("Drone 9");
		MenuItem Drone10 = MenuItem("Drone 10");

	MenuItem MainMenu8 = MenuItem("TV Remotes");
		MenuItem TV1 = MenuItem("iPod Dock");
		MenuItem TV2 = MenuItem("AOC TV");
		MenuItem TV3 = MenuItem("Sky Box");
		MenuItem TV4 = MenuItem("Busch TV");
		MenuItem TV5 = MenuItem("Panasonic");
		MenuItem TV6 = MenuItem("JVC");
		MenuItem TV7 = MenuItem("Remote 7");
		MenuItem TV8 = MenuItem("Remote 8");
		MenuItem TV9 = MenuItem("Remote 9");
		MenuItem TV10 = MenuItem("Remote 10");

	MenuItem MainMenu9 = MenuItem("Games");
		MenuItem Game1 = MenuItem("Snake");
		MenuItem Game2 = MenuItem("Pong");
		MenuItem Game3 = MenuItem("Brick");
		MenuItem Game4 = MenuItem("Game 4");
		MenuItem Game5 = MenuItem("Game 5");
		MenuItem Game6 = MenuItem("Game 6");
		MenuItem Game7 = MenuItem("Game 7");
		MenuItem Game8 = MenuItem("Game 8");
		MenuItem Game9 = MenuItem("Game 9");
		MenuItem Game10 = MenuItem("Game 10");

	MenuItem MainMenu10 = MenuItem("Tools");
		MenuItem Tools1 = MenuItem("Pot & Enc Test");
		MenuItem Tools2 = MenuItem("Button Test");
		MenuItem Tools3 = MenuItem("NRF24 Details");
		MenuItem Tools4 = MenuItem("XBee Test");
		MenuItem Tools5 = MenuItem("Display Test");
		MenuItem Tools6 = MenuItem("Touchscreen Test");
		MenuItem Tools7 = MenuItem("Tool 7");
		MenuItem Tools8 = MenuItem("Tool 8");
		MenuItem Tools9 = MenuItem("Tool 9");
		MenuItem Tools10 = MenuItem("Tool 10");

void menuUseEvent(MenuUseEvent used)
{
	if(used.item == Farm1){Vehicle_Active = 1; Current_Vehicle_Name = {Farm1.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm2){Vehicle_Active = 2; Current_Vehicle_Name = {Farm2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm3){Vehicle_Active = 3; Current_Vehicle_Name = {Farm3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm4){Vehicle_Active = 4; Current_Vehicle_Name = {Farm4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm5){Vehicle_Active = 5; Current_Vehicle_Name = {Farm5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm6){Vehicle_Active = 6; Current_Vehicle_Name = {Farm6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm7){Vehicle_Active = 7; Current_Vehicle_Name = {Farm7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm8){Vehicle_Active = 8; Current_Vehicle_Name = {Farm8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm9){Vehicle_Active = 9; Current_Vehicle_Name = {Farm9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Farm10){Vehicle_Active = 10; Current_Vehicle_Name = {Farm10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}

	if(used.item == Haulage1){Vehicle_Active = 11; Current_Vehicle_Name = {Haulage1.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage2){Vehicle_Active = 12; Current_Vehicle_Name = {Haulage2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage3){Vehicle_Active = 13; Current_Vehicle_Name = {Haulage3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage4){Vehicle_Active = 14; Current_Vehicle_Name = {Haulage4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage5){Vehicle_Active = 15; Current_Vehicle_Name = {Haulage5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage6){Vehicle_Active = 16; Current_Vehicle_Name = {Haulage6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage7){Vehicle_Active = 17; Current_Vehicle_Name = {Haulage7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage8){Vehicle_Active = 18; Current_Vehicle_Name = {Haulage8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage9){Vehicle_Active = 19; Current_Vehicle_Name = {Haulage9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Haulage10){Vehicle_Active = 20; Current_Vehicle_Name = {Haulage10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}

	if(used.item == Construction1){Vehicle_Active = 21; Current_Vehicle_Name = {Construction1.getName()}; Com_Method = 0; RC_Library_TC.Start_Hitachi();}
	if(used.item == Construction2){Vehicle_Active = 22; Current_Vehicle_Name = {Construction2.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction3){Vehicle_Active = 23; Current_Vehicle_Name = {Construction3.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction4){Vehicle_Active = 24; Current_Vehicle_Name = {Construction4.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction5){Vehicle_Active = 25; Current_Vehicle_Name = {Construction5.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction6){Vehicle_Active = 26; Current_Vehicle_Name = {Construction6.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction7){Vehicle_Active = 27; Current_Vehicle_Name = {Construction7.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction8){Vehicle_Active = 28; Current_Vehicle_Name = {Construction8.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction9){Vehicle_Active = 29; Current_Vehicle_Name = {Construction9.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}
	if(used.item == Construction10){Vehicle_Active = 30; Current_Vehicle_Name = {Construction10.getName()}; Com_Method = 0; RC_Library_TC.Start_Vehicle();}

	if(used.item == SikuControl1){Vehicle_Active = 31; Current_Vehicle_Name = {SikuControl1.getName()}; Com_Method = 2; RC_Library_TC.Start_Vehicle();}
	if(used.item == SikuControl2){Vehicle_Active = 32; Current_Vehicle_Name = {SikuControl2.getName()}; Com_Method = 2; RC_Library_TC.Start_Vehicle();}
	if(used.item == SikuControl3){Vehicle_Active = 33; Current_Vehicle_Name = {SikuControl3.getName()}; Com_Method = 2; RC_Library_TC.Start_Vehicle();}
	if(used.item == SikuControl4){Vehicle_Active = 34; Current_Vehicle_Name = {SikuControl4.getName()}; Com_Method = 2; RC_Library_TC.Start_Vehicle();}

	if(used.item == IRHeli1){IR_Heli_Active = 1; RC_Library_TC.Start_IR_Heli();}
	if(used.item == IRHeli2){IR_Heli_Active = 2; RC_Library_TC.Start_IR_Heli();}

	if(used.item == Drone3){Humanoid_Active = 1; RC_Library_TC.Start_Humanoid();}

	if(used.item == TV1){TV_Active = 1; RC_Library_TC.Start_TV();}
	if(used.item == TV2){TV_Active = 2; RC_Library_TC.Start_TV();}
	if(used.item == TV3){TV_Active = 3; RC_Library_TC.Start_TV();}
	if(used.item == TV4){TV_Active = 4; RC_Library_TC.Start_TV();}
	if(used.item == TV5){TV_Active = 5; RC_Library_TC.Start_TV();}
	if(used.item == TV6){TV_Active = 6; RC_Library_TC.Start_TV();}
	if(used.item == TV7){TV_Active = 7; RC_Library_TC.Start_TV();}
	if(used.item == TV8){TV_Active = 8; RC_Library_TC.Start_TV();}
	if(used.item == TV9){TV_Active = 9; RC_Library_TC.Start_TV();}
	if(used.item == TV10){TV_Active = 10; RC_Library_TC.Start_TV();}

	if(used.item == Game1){Game_Active = 1; RC_Library_TC.Start_Game();}
	if(used.item == Game2){Game_Active = 2; RC_Library_TC.Start_Game();}
	if(used.item == Game3){Game_Active = 3; RC_Library_TC.Start_Game();}
	if(used.item == Game4){Game_Active = 4; RC_Library_TC.Start_Game();}
	if(used.item == Game5){Game_Active = 5; RC_Library_TC.Start_Game();}
	if(used.item == Game6){Game_Active = 6; RC_Library_TC.Start_Game();}
	if(used.item == Game7){Game_Active = 7; RC_Library_TC.Start_Game();}
	if(used.item == Game8){Game_Active = 8; RC_Library_TC.Start_Game();}
	if(used.item == Game9){Game_Active = 9; RC_Library_TC.Start_Game();}
	if(used.item == Game10){Game_Active = 10; RC_Library_TC.Start_Game();}

	if(used.item == Tools1){Tool_Active = 1; RC_Library_TC.Select_Tool();}
	if(used.item == Tools2){Tool_Active = 2; RC_Library_TC.Select_Tool();}
	if(used.item == Tools3){Tool_Active = 3; RC_Library_TC.Select_Tool();}
	if(used.item == Tools4){Tool_Active = 4; RC_Library_TC.Select_Tool();}
	if(used.item == Tools5){Tool_Active = 5; RC_Library_TC.Select_Tool();}
	if(used.item == Tools6){Tool_Active = 6; RC_Library_TC.Select_Tool();}
}


/*
	This is an important function
	Here we get a notification whenever the user changes the menu
	That is, when the menu is navigated
*/
void menuChangeEvent(MenuChangeEvent changed)
{
	// Main Menu
	if(changed.to.getName() == MainMenu1.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	if(changed.to.getName() == MainMenu2.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	if(changed.to.getName() == MainMenu3.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	if(changed.to.getName() == MainMenu4.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	if(changed.to.getName() == MainMenu5.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	if(changed.to.getName() == MainMenu6.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	if(changed.to.getName() == MainMenu7.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	if(changed.to.getName() == MainMenu8.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	if(changed.to.getName() == MainMenu9.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	if(changed.to.getName() == MainMenu10.getName()){Menu_Level = 0;if(Menu_Change != 0){RC_Library_TC.MenuBackground1();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 1
	if(changed.to.getName() == Farm1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	if(changed.to.getName() == Farm2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	if(changed.to.getName() == Farm3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	if(changed.to.getName() == Farm4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	if(changed.to.getName() == Farm5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	if(changed.to.getName() == Farm6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	if(changed.to.getName() == Farm7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	if(changed.to.getName() == Farm8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	if(changed.to.getName() == Farm9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	if(changed.to.getName() == Farm10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground2();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 2
	if(changed.to.getName() == Haulage1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Haulage2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Haulage3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Haulage4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Haulage5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Haulage6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Haulage7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Haulage8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Haulage9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Haulage10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground3();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 3
	if(changed.to.getName() == Construction1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Construction2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Construction3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Construction4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Construction5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Construction6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Construction7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Construction8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Construction9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Construction10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground4();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 4
	if(changed.to.getName() == Trailer1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Trailer2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Trailer3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Trailer4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Trailer5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Trailer6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Trailer7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Trailer8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Trailer9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Trailer10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground5();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 5
	if(changed.to.getName() == SikuControl1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground6();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == SikuControl2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground6();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == SikuControl3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground6();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == SikuControl4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground6();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}

	//Sub Menu 6
	if(changed.to.getName() == IRHeli1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == IRHeli2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == IRHeli3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == IRHeli4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == IRHeli5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == IRHeli6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == IRHeli7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == IRHeli8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == IRHeli9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == IRHeli10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground7();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 7
	if(changed.to.getName() == Drone1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Drone2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Drone3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Drone4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Drone5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Drone6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Drone7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Drone8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Drone9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Drone10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground8();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 8
	if(changed.to.getName() == TV1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == TV2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == TV3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == TV4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == TV5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == TV6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == TV7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == TV8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == TV9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == TV10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground9();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 9
	if(changed.to.getName() == Game1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Game2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Game3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Game4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Game5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Game6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Game7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Game8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Game9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Game10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground10();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}

	//Sub Menu 10
	if(changed.to.getName() == Tools1.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(1);}
	else if(changed.to.getName() == Tools2.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(2);}
	else if(changed.to.getName() == Tools3.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(3);}
	else if(changed.to.getName() == Tools4.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(4);}
	else if(changed.to.getName() == Tools5.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(5);}
	else if(changed.to.getName() == Tools6.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(6);}
	else if(changed.to.getName() == Tools7.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(7);}
	else if(changed.to.getName() == Tools8.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(8);}
	else if(changed.to.getName() == Tools9.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(9);}
	else if(changed.to.getName() == Tools10.getName()){Menu_Level = 1;if(Menu_Change != 0){RC_Library_TC.MenuBackground11();Menu_Change = 0;}RC_Library_TC.MenuPosition(10);}
}


//this controls the menu backend and the event generation
MenuBackend menu = MenuBackend(menuUseEvent,menuChangeEvent);





/*******************************************************************/
/*          RC Farm and Construction Vehicle Functions             */
/*******************************************************************/

RCTractorControllerV2::RCTractorControllerV2(int Controller_Ver)
{
	_JX_L = A12;
	_JY_L = A14;
	_S_Left_Joy = 36;
	_JX_R = A15;
	_JY_R = A13;
	_S_Right_Joy = 12;

	_S_EN1 = 40;
	_S_EN2 = 31;

	_VQS_V1 = A10;
	_VQS_V2 = A9;
	_VQS_V3 = A8;
	_VQS_V4 = A7;
	_VQS_V5 = A6;
	if(Controller_Ver==20){_VQS_V6 = 6;}else if(Controller_Ver==21){_VQS_V6 = 8;}
	if(Controller_Ver==20){_VQS_V7 = 5;}else if(Controller_Ver==21){_VQS_V7 = 7;}
	if(Controller_Ver==20){_VQS_V8 = 4;}else if(Controller_Ver==21){_VQS_V8 = 6;}
	if(Controller_Ver==20){_VQS_V9 = 3;}else if(Controller_Ver==21){_VQS_V9 = 5;}
	if(Controller_Ver==20){_VQS_V10 = 2;}else if(Controller_Ver==21){_VQS_V10 = 4;}

	_S_DB = 35;
	_S_FB = 34;
	_S_WL = 37;

	if(Controller_Ver==20){_S_F1 = 13;}else if(Controller_Ver==21){_S_F1 = 14;}
	if(Controller_Ver==20){_S_F2 = 10;}else if(Controller_Ver==21){_S_F2 = 15;}
	if(Controller_Ver==20){_S_F3 = 11;}else if(Controller_Ver==21){_S_F3 = 11;}

	_S_Up = 32;
	_S_Down = 33;
	if(Controller_Ver==20){_S_Select = 8;}else if(Controller_Ver==21){_S_Select = 10;}
	if(Controller_Ver==20){_S_Back = 7;}else if(Controller_Ver==21){_S_Back = 9;}

	_DIP1 = 46;
	_DIP2 = 45;
	_DIP3 = 44;
	_DIP4 = 43;
	_DIP5 = 42;

	_IR_LED = 47;

	_LED_Data = 0;
	_FUNC_Data = 0;
	_Vehicle_No = 0;

	pinMode(_S_Left_Joy, INPUT_PULLUP);
	pinMode(_S_Right_Joy, INPUT_PULLUP);
	pinMode(_VQS_V1, INPUT_PULLUP);
	pinMode(_VQS_V2, INPUT_PULLUP);
	pinMode(_VQS_V3, INPUT_PULLUP);
	pinMode(_VQS_V4, INPUT_PULLUP);
	pinMode(_VQS_V5, INPUT_PULLUP);
	pinMode(_VQS_V6, INPUT_PULLUP);
	pinMode(_VQS_V7, INPUT_PULLUP);
	pinMode(_VQS_V8, INPUT_PULLUP);
	pinMode(_VQS_V9, INPUT_PULLUP);
	pinMode(_VQS_V10, INPUT_PULLUP);
	pinMode(_S_DB, INPUT_PULLUP);
	pinMode(_S_FB, INPUT_PULLUP);
	pinMode(_S_WL, INPUT_PULLUP);
	pinMode(_S_F1, INPUT_PULLUP);
	pinMode(_S_F2, INPUT_PULLUP);
	pinMode(_S_F3, INPUT_PULLUP);
	pinMode(_S_EN1, INPUT_PULLUP);
	pinMode(_S_EN2, INPUT_PULLUP);
	pinMode(_S_Up, INPUT_PULLUP);
	pinMode(_S_Down, INPUT_PULLUP);
	pinMode(_S_Select, INPUT_PULLUP);
	pinMode(_S_Back, INPUT_PULLUP);
	pinMode(_DIP1, INPUT_PULLUP);
	pinMode(_DIP2, INPUT_PULLUP);
	pinMode(_DIP3, INPUT_PULLUP);
	pinMode(_DIP4, INPUT_PULLUP);
	pinMode(_DIP5, INPUT_PULLUP);

	pinMode(_IR_LED, OUTPUT);

	if(Controller_Ver==20){Encoder1.Setup(20, 18);}else if(Controller_Ver==21){Encoder1.Setup(2, 3);}
	if(Controller_Ver==20){Encoder2.Setup(21, 19);}else if(Controller_Ver==21){Encoder2.Setup(18, 19);}


}

void RCTractorControllerV2::Setup(int Control_Num){

Controller_Num = Control_Num;

command[11] = Controller_Num;

	  Serial.begin(9600);

	Radio_Attached = radio.begin();

	// RF24_PA_MIN, RF24_PA_LOW, RF24_PA_HIGH and RF24_PA_MAX
	radio.setPALevel(RF24_PA_LOW);

	// RF24_250KBPS for 250kbs, RF24_1MBPS for 1Mbps, or RF24_2MBPS for 2Mbps
	//radio.setDataRate(RF24_250KBPS);

	radio.openWritingPipe(Controller_Send[Controller_Num]);

    radio.openReadingPipe(1,Controller_Listen[Controller_Num]);

	// Denomiator used in voltage measurements
	Denominator = (float)850/(3300 + 850);

	// Initialize all the measurement readings to 0:
	for (int thisReading = 0; thisReading < numReadings; thisReading++) {
		Controller_V_Readings[thisReading] = 0;
		Vehicle_V_Readings[thisReading] = 0;
	}

	// Menu Setup
	//add the file menu to the menu root
	menu.getRoot().add(MainMenu1);
	//setup the settings menu item
	MainMenu1.addRight(Farm1);
	//we want looping both up and down
		Farm1.addBefore(Farm10);
		Farm1.addAfter(Farm2);
		Farm2.addAfter(Farm3);
		Farm3.addAfter(Farm4);
		Farm4.addAfter(Farm5);
		Farm5.addAfter(Farm6);
		Farm6.addAfter(Farm7);
		Farm7.addAfter(Farm8);
		Farm8.addAfter(Farm9);
		Farm9.addAfter(Farm10);
		Farm10.addAfter(Farm1);

		//we want a left movement to pint to settings from anywhere
		Farm1.addLeft(MainMenu1);
		Farm2.addLeft(MainMenu1);
		Farm3.addLeft(MainMenu1);
		Farm4.addLeft(MainMenu1);
		Farm5.addLeft(MainMenu1);
		Farm6.addLeft(MainMenu1);
		Farm7.addLeft(MainMenu1);
		Farm8.addLeft(MainMenu1);
		Farm9.addLeft(MainMenu1);
		Farm10.addLeft(MainMenu1);

	MainMenu1.addBefore(MainMenu10);
	MainMenu1.addAfter(MainMenu2);

	MainMenu2.addRight(Haulage1);
	//we want looping both up and down
		Haulage1.addBefore(Haulage10);
		Haulage1.addAfter(Haulage2);
		Haulage2.addAfter(Haulage3);
		Haulage3.addAfter(Haulage4);
		Haulage4.addAfter(Haulage5);
		Haulage5.addAfter(Haulage6);
		Haulage6.addAfter(Haulage7);
		Haulage7.addAfter(Haulage8);
		Haulage8.addAfter(Haulage9);
		Haulage9.addAfter(Haulage10);
		Haulage10.addAfter(Haulage1);

		//we want a left movement to pint to settings from anywhere
		Haulage1.addLeft(MainMenu2);
		Haulage2.addLeft(MainMenu2);
		Haulage3.addLeft(MainMenu2);
		Haulage4.addLeft(MainMenu2);
		Haulage5.addLeft(MainMenu2);
		Haulage6.addLeft(MainMenu2);
		Haulage7.addLeft(MainMenu2);
		Haulage8.addLeft(MainMenu2);
		Haulage9.addLeft(MainMenu2);
		Haulage10.addLeft(MainMenu2);

	MainMenu2.addAfter(MainMenu3);

	MainMenu3.addRight(Construction1);
	//we want looping both up and down
		Construction1.addBefore(Construction10);
		Construction1.addAfter(Construction2);
		Construction2.addAfter(Construction3);
		Construction3.addAfter(Construction4);
		Construction4.addAfter(Construction5);
		Construction5.addAfter(Construction6);
		Construction6.addAfter(Construction7);
		Construction7.addAfter(Construction8);
		Construction8.addAfter(Construction9);
		Construction9.addAfter(Construction10);
		Construction10.addAfter(Construction1);

		//we want a left movement to pint to settings from anywhere
		Construction1.addLeft(MainMenu3);
		Construction2.addLeft(MainMenu3);
		Construction3.addLeft(MainMenu3);
		Construction4.addLeft(MainMenu3);
		Construction5.addLeft(MainMenu3);
		Construction6.addLeft(MainMenu3);
		Construction7.addLeft(MainMenu3);
		Construction8.addLeft(MainMenu3);
		Construction9.addLeft(MainMenu3);
		Construction10.addLeft(MainMenu3);

	MainMenu3.addAfter(MainMenu4);

	MainMenu4.addRight(Trailer1);
		//we want looping both up and down
		Trailer1.addBefore(Trailer10);
		Trailer1.addAfter(Trailer2);
		Trailer2.addAfter(Trailer3);
		Trailer3.addAfter(Trailer4);
		Trailer4.addAfter(Trailer5);
		Trailer5.addAfter(Trailer6);
		Trailer6.addAfter(Trailer7);
		Trailer7.addAfter(Trailer8);
		Trailer8.addAfter(Trailer9);
		Trailer9.addAfter(Trailer10);
		Trailer10.addAfter(Trailer1);

		//we want a left movement to pint to settings from anywhere
		Trailer1.addLeft(MainMenu4);
		Trailer2.addLeft(MainMenu4);
		Trailer3.addLeft(MainMenu4);
		Trailer4.addLeft(MainMenu4);
		Trailer5.addLeft(MainMenu4);
		Trailer6.addLeft(MainMenu4);
		Trailer7.addLeft(MainMenu4);
		Trailer8.addLeft(MainMenu4);
		Trailer9.addLeft(MainMenu4);
		Trailer10.addLeft(MainMenu4);

	MainMenu4.addAfter(MainMenu5);

	MainMenu5.addRight(SikuControl1);
		//we want looping both up and down
		SikuControl1.addBefore(SikuControl4);
		SikuControl1.addAfter(SikuControl2);
		SikuControl2.addAfter(SikuControl3);
		SikuControl3.addAfter(SikuControl4);
		SikuControl4.addAfter(SikuControl1);

		//we want a left movement to pint to settings from anywhere
		SikuControl1.addLeft(MainMenu5);
		SikuControl2.addLeft(MainMenu5);
		SikuControl3.addLeft(MainMenu5);
		SikuControl4.addLeft(MainMenu5);

	MainMenu5.addAfter(MainMenu6);

	MainMenu6.addRight(IRHeli1);
		//we want looping both up and down
		IRHeli1.addBefore(IRHeli10);
		IRHeli1.addAfter(IRHeli2);
		IRHeli2.addAfter(IRHeli3);
		IRHeli3.addAfter(IRHeli4);
		IRHeli4.addAfter(IRHeli5);
		IRHeli5.addAfter(IRHeli6);
		IRHeli6.addAfter(IRHeli7);
		IRHeli7.addAfter(IRHeli8);
		IRHeli8.addAfter(IRHeli9);
		IRHeli9.addAfter(IRHeli10);
		IRHeli10.addAfter(IRHeli1);

		//we want a left movement to pint to settings from anywhere
		IRHeli1.addLeft(MainMenu6);
		IRHeli2.addLeft(MainMenu6);
		IRHeli3.addLeft(MainMenu6);
		IRHeli4.addLeft(MainMenu6);
		IRHeli5.addLeft(MainMenu6);
		IRHeli6.addLeft(MainMenu6);
		IRHeli7.addLeft(MainMenu6);
		IRHeli8.addLeft(MainMenu6);
		IRHeli9.addLeft(MainMenu6);
		IRHeli10.addLeft(MainMenu6);

	MainMenu6.addAfter(MainMenu7);

	MainMenu7.addRight(Drone1);
		//we want looping both up and down
		Drone1.addBefore(Drone10);
		Drone1.addAfter(Drone2);
		Drone2.addAfter(Drone3);
		Drone3.addAfter(Drone4);
		Drone4.addAfter(Drone5);
		Drone5.addAfter(Drone6);
		Drone6.addAfter(Drone7);
		Drone7.addAfter(Drone8);
		Drone8.addAfter(Drone9);
		Drone9.addAfter(Drone10);
		Drone10.addAfter(Drone1);

		//we want a left movement to pint to settings from anywhere
		Drone1.addLeft(MainMenu7);
		Drone2.addLeft(MainMenu7);
		Drone3.addLeft(MainMenu7);
		Drone4.addLeft(MainMenu7);
		Drone5.addLeft(MainMenu7);
		Drone6.addLeft(MainMenu7);
		Drone7.addLeft(MainMenu7);
		Drone8.addLeft(MainMenu7);
		Drone9.addLeft(MainMenu7);
		Drone10.addLeft(MainMenu7);

	MainMenu7.addAfter(MainMenu8);

	MainMenu8.addRight(TV1);
		//we want looping both up and down
		TV1.addBefore(TV10);
		TV1.addAfter(TV2);
		TV2.addAfter(TV3);
		TV3.addAfter(TV4);
		TV4.addAfter(TV5);
		TV5.addAfter(TV6);
		TV6.addAfter(TV7);
		TV7.addAfter(TV8);
		TV8.addAfter(TV9);
		TV9.addAfter(TV10);
		TV10.addAfter(TV1);

		//we want a left movement to pint to settings from anywhere
		TV1.addLeft(MainMenu8);
		TV2.addLeft(MainMenu8);
		TV3.addLeft(MainMenu8);
		TV4.addLeft(MainMenu8);
		TV5.addLeft(MainMenu8);
		TV6.addLeft(MainMenu8);
		TV7.addLeft(MainMenu8);
		TV8.addLeft(MainMenu8);
		TV9.addLeft(MainMenu8);
		TV10.addLeft(MainMenu8);

	MainMenu8.addAfter(MainMenu9);

	MainMenu9.addRight(Game1);
		//we want looping both up and down
		Game1.addBefore(Game10);
		Game1.addAfter(Game2);
		Game2.addAfter(Game3);
		Game3.addAfter(Game4);
		Game4.addAfter(Game5);
		Game5.addAfter(Game6);
		Game6.addAfter(Game7);
		Game7.addAfter(Game8);
		Game8.addAfter(Game9);
		Game9.addAfter(Game10);
		Game10.addAfter(Game1);

		//we want a left movement to pint to settings from anywhere
		Game1.addLeft(MainMenu9);
		Game2.addLeft(MainMenu9);
		Game3.addLeft(MainMenu9);
		Game4.addLeft(MainMenu9);
		Game5.addLeft(MainMenu9);
		Game6.addLeft(MainMenu9);
		Game7.addLeft(MainMenu9);
		Game8.addLeft(MainMenu9);
		Game9.addLeft(MainMenu9);
		Game10.addLeft(MainMenu9);

	MainMenu9.addAfter(MainMenu10);

	MainMenu10.addRight(Tools1);
		//we want looping both up and down
		Tools1.addBefore(Tools10);
		Tools1.addAfter(Tools2);
		Tools2.addAfter(Tools3);
		Tools3.addAfter(Tools4);
		Tools4.addAfter(Tools5);
		Tools5.addAfter(Tools6);
		Tools6.addAfter(Tools7);
		Tools7.addAfter(Tools8);
		Tools8.addAfter(Tools9);
		Tools9.addAfter(Tools10);
		Tools10.addAfter(Tools1);

		//we want a left movement to pint to settings from anywhere
		Tools1.addLeft(MainMenu10);
		Tools2.addLeft(MainMenu10);
		Tools3.addLeft(MainMenu10);
		Tools4.addLeft(MainMenu10);
		Tools5.addLeft(MainMenu10);
		Tools6.addLeft(MainMenu10);
		Tools7.addLeft(MainMenu10);
		Tools8.addLeft(MainMenu10);
		Tools9.addLeft(MainMenu10);
		Tools10.addLeft(MainMenu10);

	MainMenu10.addAfter(MainMenu1);

	// Initialise the encoders
	Encoder1.write(25);
	Encoder2.write(25);

	// Initialise the TFT and SD Card
	tft.reset();

	identifier = tft.readID();

Serial.print("Iden: '");
Serial.print(identifier,HEX);
Serial.print("'");

	// If TFT doesn't work try commenting out the above line and trying each of the following one by one
	//identifier = 0x9325;
	//identifier = 0x9328;
	//identifier = 0x7575;
	//identifier = 0x8357;
	//identifier = 0x9341;
	//identifier = 0x9327;
	//identifier = 0x0154;
	//identifier = 0x9488;
	//identifier = 0x9329;
	//identifier = 0x1520;
 if (identifier == 0x00D3 || identifier == 0xD3D3) identifier = 0x9481; // write-only shield
    if (identifier == 0xFFFF) identifier = 0x9341; // serial
    if (identifier == 0x0000) identifier = 0x1520;
    //identifier = 0x9329;                             // force ID
    tft.begin(identifier);
	if(identifier == 0x1520 || identifier == 0x9341){
	tft.setRotation(2);
}
else{
	tft.setRotation(0);
}

	SD.begin(SD_CS);

	//bmpDraw("logo.bmp", 0, 60);

	//delay(1000);
	Check_Joys();
	command[6]=125;
	command[8]=125;
	Check_Encoders();
	Menu_Change = 1;
	menu.moveDown();
}

void RCTractorControllerV2::Update(){

    Menu_Update();

}

void RCTractorControllerV2::Check_Joys(){
	// Check Joy Stick Potentiometers
	_Left_Joy_X = analogRead(_JX_L);
	_Left_Joy_X= map(_Left_Joy_X, 0, 1024, 0, 255);
	_Left_Joy_Y = analogRead(_JY_L);
	_Left_Joy_Y= map(_Left_Joy_Y, 0, 1024, 0, 255);
	_Right_Joy_X = analogRead(_JX_R);
	_Right_Joy_X= map(_Right_Joy_X, 0, 1024, 0, 255);
	_Right_Joy_Y = analogRead(_JY_R);
	_Right_Joy_Y= map(_Right_Joy_Y, 0, 1024, 0, 255);

	if(_Left_Joy_X != _Left_Joy_X_Old || _Left_Joy_Y != _Left_Joy_Y_Old || _Right_Joy_X != _Right_Joy_X_Old || _Right_Joy_Y != _Right_Joy_Y_Old ){

		Throttle_Position = _Left_Joy_Y;

		if(Vehicle_Gear == 1){
			_Left_Joy_Y = map(Throttle_Position, 0, 255, 84, 171);
		}
		else if(Vehicle_Gear == 2){
			_Left_Joy_Y = map(Throttle_Position, 0, 255, 42, 213);
		}

		command[1]=_Left_Joy_X;
		command[2]=_Left_Joy_Y;
		command[3]=_Right_Joy_X;
		command[4]=_Right_Joy_Y;

		_Left_Joy_X_Old = _Left_Joy_X;
		_Left_Joy_Y_Old = _Left_Joy_Y;
		_Right_Joy_X_Old = _Right_Joy_X;
		_Right_Joy_Y_Old = _Right_Joy_Y;
	}
}


void RCTractorControllerV2::Check_Encoders(){
	Encoder1NewPosition = Encoder1.read();
	Encoder2NewPosition = Encoder2.read();

	if (Encoder1NewPosition != Encoder1OldPosition) {
		if(Encoder1NewPosition<0){
			Encoder1NewPosition=0;
			Encoder1.write(Encoder1NewPosition);
		}
		if(Encoder1NewPosition>Encoder1_Limit){
			Encoder1NewPosition=Encoder1_Limit;
			Encoder1.write(Encoder1NewPosition);
		}
		Encoder1OldPosition = Encoder1NewPosition;
		if (bitRead(EN_Mode,0) == 1 && bitRead(EN_Mode,1) == 0){
		command[5]=(Encoder1NewPosition*(255/Encoder1_Limit));
		}
		else {
		command[6]=(Encoder1NewPosition*(255/Encoder1_Limit));
		}
	}

	if (Encoder2NewPosition != Encoder2OldPosition) {
		if(Encoder2NewPosition<0){
			Encoder2NewPosition=0;
			Encoder2.write(Encoder2NewPosition);
		}
		if(Encoder2NewPosition>51){
			Encoder2NewPosition=51;
			Encoder2.write(Encoder2NewPosition);
		}
		Encoder2OldPosition = Encoder2NewPosition;

		if (bitRead(EN_Mode,2) == 1 && bitRead(EN_Mode,3) == 0){
			command[7]=(Encoder2NewPosition*5);
		}
		else {
			command[8]=(Encoder2NewPosition*5);
		}
	}

	if ((digitalRead(_S_EN1))==LOW)
	{
		if (bitRead(EN_Mode,0) == 1 && bitRead(EN_Mode,1) == 0){
			bitClear(EN_Mode, 0);
			bitSet(EN_Mode, 1);
			Encoder1.write(command[6]/5);
		}
		else {
			bitSet(EN_Mode, 0);
			bitClear(EN_Mode, 1);
			Encoder1.write(command[5]/5);
		}

		// Wait for switch to be released
		while ((digitalRead(_S_EN1)) == LOW){}
	}

	if ((digitalRead(_S_EN2))==LOW)
	{
		if (bitRead(EN_Mode,2) == 1 && bitRead(EN_Mode,3) == 0){
			bitClear(EN_Mode, 2);
			bitSet(EN_Mode, 3);
			Encoder2.write(command[8]/5);
		}
		else {
			bitSet(EN_Mode, 2);
			bitClear(EN_Mode, 3);
			Encoder2.write(command[7]/5);
		}

		// Wait for switch to be released
		while ((digitalRead(_S_EN2)) == LOW){}
	}
}

void RCTractorControllerV2::Check_Light_Buttons(){
	// Check Joystick Switches
	if (digitalRead(_S_Left_Joy) == LOW || digitalRead(_S_Right_Joy) == LOW)
	{
		delay(10); // Allow time for both switches to activate
		if (digitalRead(_S_Left_Joy) == LOW && digitalRead(_S_Right_Joy) == LOW)
		{
			if((bitRead(_LED_Data, 0))!=1 && (bitRead(_LED_Data, 1))!=1){
				bitWrite(_LED_Data, 0, 1);  // Turn on Hazard Lights
				bitWrite(_LED_Data, 1, 1);
			}
			else{
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Left_Joy) == LOW || digitalRead(_S_Right_Joy) == LOW){}
		}
		else if (digitalRead(_S_Right_Joy) == LOW)
		{
			if ((bitRead(_LED_Data, 0))!=1){
				bitWrite(_LED_Data, 0, 1);  // Command to turn on right indicator
			}
			else {
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Right_Joy) == LOW){}
		}
		else if (digitalRead(_S_Left_Joy) == LOW)
		{
			if ((bitRead(_LED_Data, 1))!=1){
				bitWrite(_LED_Data, 1, 1);  // Command to turn on right indicator
			}
			else {
				bitWrite(_LED_Data, 0, 0);  // Turn off Indicators
				bitWrite(_LED_Data, 1, 0);
			}

			// Wait for switch to be released
			while (digitalRead(_S_Left_Joy) == LOW){}
		}
	}

	if ((digitalRead(_S_DB))==LOW)
	{
		if ((bitRead(_LED_Data, 2))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 0);  // Turn off Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_DB)) == LOW){}
	}

	if (digitalRead(_S_FB) == LOW && bitRead(_LED_Data, 2)!=0)
	{
		if ((bitRead(_LED_Data, 3))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 0);  // Turn off Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_FB)) == LOW){}
	}

	if ((digitalRead(_S_WL))==LOW && bitRead(_LED_Data, 2)!=0)
	{
		if ((bitRead(_LED_Data, 4))!=1){
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 1);  // Turn on Work Lights
		}
		else {
			bitWrite(_LED_Data, 2, 1);  // Turn on Dips
			bitWrite(_LED_Data, 3, 1);  // Turn on Full Beams
			bitWrite(_LED_Data, 4, 0);  // Turn off Work Lights
		}

		// Wait for switch to be released
		while ((digitalRead(_S_WL)) == LOW){}
	}
	command[9] = _LED_Data;
}


void RCTractorControllerV2::Check_Func_Buttons(){

	if ((digitalRead(_S_F1))==LOW)
	{
    	if ((bitRead(_FUNC_Data, 0))!=1){
      		bitWrite(_FUNC_Data, 0, 1);  // Turn on Function 1
    	}
    	else {
      		bitWrite(_FUNC_Data, 0, 0);  // Turn off Function 1
    	}

    	// Wait for switch to be released
    	while ((digitalRead(_S_F1)) == LOW){}
	}


	if ((digitalRead(_S_F2))==LOW)
  	{

    	if ((bitRead(_FUNC_Data, 1))!=1){
      		bitWrite(_FUNC_Data, 1, 1);  // Turn on Function 1
    	}
    	else {
      		bitWrite(_FUNC_Data, 1, 0);  // Turn off Function 1
    	}

    	// Wait for switch to be released
    	while ((digitalRead(_S_F2)) == LOW){}
  	}

  	if ((digitalRead(_S_F3))==LOW)
    {

		if ((bitRead(_FUNC_Data, 2))!=1){
        	bitWrite(_FUNC_Data, 2, 1);  // Turn on Function 1
      	}
      	else {
        	bitWrite(_FUNC_Data, 2, 0);  // Turn off Function 1
      	}

      	// Wait for switch to be released
      	while ((digitalRead(_S_F3)) == LOW){}
	}
	command[10] = _FUNC_Data;
}


void RCTractorControllerV2::Menu_Update(){
	if ((digitalRead(_S_Up))==LOW)
	{
		menu.moveUp();

		// Wait for switch to be released
		while ((digitalRead(_S_Up)) == LOW){}
		delay(25);
	}

	if ((digitalRead(_S_Down))==LOW)
	{
		menu.moveDown();

		// Wait for switch to be released
		while ((digitalRead(_S_Down)) == LOW){}
				delay(25);
	}

	if ((digitalRead(_S_Back))==LOW)
	{
		if(Menu_Level == 1){
			menu.moveLeft();
			Menu_Change = 1;
			menu.moveDown();
			menu.moveUp();
		}

		// Wait for switch to be released
		while ((digitalRead(_S_Back)) == LOW){}
	}

	if ((digitalRead(_S_Select))==LOW)
	{
		if(Menu_Level == 0){
			menu.moveRight();
			Menu_Change = 1;
			menu.moveDown();
		}
		else if(Menu_Level == 1){
			menu.use();
		}

		// Wait for switch to be released
		while ((digitalRead(_S_Select)) == LOW){}
	}
}

void RCTractorControllerV2::Check_Fast_Select(){

	if ((digitalRead(_VQS_V1))==LOW)
  {
	  _V_ID = 0;
	  // Wait for switch to be released
	      while ((digitalRead(_VQS_V1)) == LOW)
	      {

    }
  }

	else if ((digitalRead(_VQS_V2))==LOW)
  {
	  _V_ID = 1;
	  // Wait for switch to be released
	      while ((digitalRead(_VQS_V2)) == LOW)
	      {

    }
  }

  	else if ((digitalRead(_VQS_V3))==LOW)
    {
		_V_ID = 2;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V3)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V4))==LOW)
    {
		_V_ID = 3;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V4)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V5))==LOW)
    {
		_V_ID = 4;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V5)) == LOW)
  	      {

      }
  }

  	else if ((digitalRead(_VQS_V6))==LOW)
    {
		_V_ID = 5;
  	  // Wait for switch to be released
  	      while ((digitalRead(_VQS_V6)) == LOW)
  	      {

      }
  }

    	else if ((digitalRead(_VQS_V7))==LOW)
      {
  		_V_ID = 6;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V7)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V8))==LOW)
      {
  		_V_ID = 7;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V8)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V9))==LOW)
      {
  		_V_ID = 8;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V9)) == LOW)
    	      {

        }
    }

    	else if ((digitalRead(_VQS_V10))==LOW)
      {
  		_V_ID = 9;
    	  // Wait for switch to be released
    	      while ((digitalRead(_VQS_V10)) == LOW)
    	      {

        }
  }

//command[0] = _V_ID;

}

/*******************************************************************/
/*       Functions to Test Controller and Vehicle Voltages         */
/*******************************************************************/

long RCTractorControllerV2::readVcc() {
  long result;
  // Read 1.1V reference against AVcc
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1); ADCSRB &= ~_BV(MUX5);
  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Convert
  while (bit_is_set(ADCSRA,ADSC));
  result = ADCL;
  result |= ADCH<<8;
  result = (1126400L / result); // Back-calculate AVcc in mV
  return result;
}

void RCTractorControllerV2::Controller_Voltage()
{
	if(Controller_V_Measure_Period==50){
    	if(Controller_V_Readings[(numReadings-1)]==0){
			Controller_V_Measure_Period=49;
		}
		else{
    		Controller_V_Measure_Period=0;
		}

		// subtract the last reading:
		Controller_V_Total = Controller_V_Total - Controller_V_Readings[Controller_V_ReadIndex];
		// read from the sensor:
		Controller_V_Readings[Controller_V_ReadIndex] = analogRead(A5);
		// add the reading to the total:
		Controller_V_Total = Controller_V_Total + Controller_V_Readings[Controller_V_ReadIndex];
		// advance to the next position in the array:
		Controller_V_ReadIndex = Controller_V_ReadIndex + 1;

		// if we're at the end of the array...
		if (Controller_V_ReadIndex >= numReadings) {
			// ...wrap around to the beginning:
			Controller_V_ReadIndex = 0;
		}

		// calculate the average:
		Controller_V_Average = Controller_V_Total / numReadings;

		Controller_V_Average = (Controller_V_Average / 1024) * (readVcc());

		Controller_V_Average = (Controller_V_Average / Denominator)/1000;
	}
	Controller_V_Measure_Period = Controller_V_Measure_Period+1;
}


void RCTractorControllerV2::Vehicle_Voltage(long _Vehicle_V){
	Vehicle_V = _Vehicle_V;

	if(Vehicle_V_Measure_Period==25){
		if(Vehicle_V_Readings[(numReadings-1)]==0){
			Vehicle_V_Measure_Period=24;
		}
		else{
			Vehicle_V_Measure_Period=0;
		}

		Vehicle_V_Measure_Period=0;

		// subtract the last reading:
		Vehicle_V_Total = Vehicle_V_Total - Vehicle_V_Readings[Vehicle_V_ReadIndex];
		// read from the sensor:
		Vehicle_Volt =Vehicle_V;
		Vehicle_V_Readings[Vehicle_V_ReadIndex] = Vehicle_Volt/1000;
		// add the reading to the total:
		Vehicle_V_Total = Vehicle_V_Total + Vehicle_V_Readings[Vehicle_V_ReadIndex];
		// advance to the next position in the array:
		Vehicle_V_ReadIndex = Vehicle_V_ReadIndex + 1;

		// if we're at the end of the array...
		if (Vehicle_V_ReadIndex >= numReadings) {
			// ...wrap around to the beginning:
			Vehicle_V_ReadIndex = 0;
		}

		// calculate the average:
		Vehicle_V_Average = Vehicle_V_Total / numReadings;

	}
	Vehicle_V_Measure_Period = Vehicle_V_Measure_Period+1;
}

/*******************************************************************/
/*                        Menu Backgrounds                         */
/*******************************************************************/

void RCTractorControllerV2::MenuPosition(int Position) {
    tft.fillRect(5,22,7,232,BLACK);
    tft.fillTriangle(11,(27+(Position*20)),5,(22+(Position*20)),5,(32+(Position*20)),GREEN);
}

void RCTractorControllerV2::MenuBackground1() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(MainMenu1.getName());

  tft.setCursor(15, 60);
  tft.println(MainMenu2.getName());

    tft.setCursor(15, 80);
    tft.println(MainMenu3.getName());

    tft.setCursor(15, 100);
    tft.println(MainMenu4.getName());

    tft.setCursor(15, 120);
    tft.println(MainMenu5.getName());

    tft.setCursor(15, 140);
    tft.println(MainMenu6.getName());

    tft.setCursor(15, 160);
    tft.println(MainMenu7.getName());

    tft.setCursor(15, 180);
    tft.println(MainMenu8.getName());

    tft.setCursor(15, 200);
    tft.println(MainMenu9.getName());

    tft.setCursor(15, 220);
    tft.println(MainMenu10.getName());
}

void RCTractorControllerV2::MenuBackground2() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Farm1.getName());

  tft.setCursor(15, 60);
  tft.println(Farm2.getName());

    tft.setCursor(15, 80);
    tft.println(Farm3.getName());

    tft.setCursor(15, 100);
    tft.println(Farm4.getName());

    tft.setCursor(15, 120);
    tft.println(Farm5.getName());

    tft.setCursor(15, 140);
    tft.println(Farm6.getName());

    tft.setCursor(15, 160);
    tft.println(Farm7.getName());

    tft.setCursor(15, 180);
    tft.println(Farm8.getName());

    tft.setCursor(15, 200);
    tft.println(Farm9.getName());

    tft.setCursor(15, 220);
    tft.println(Farm10.getName());
}

void RCTractorControllerV2::MenuBackground3() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Haulage1.getName());

  tft.setCursor(15, 60);
  tft.println(Haulage2.getName());

    tft.setCursor(15, 80);
    tft.println(Haulage3.getName());

    tft.setCursor(15, 100);
    tft.println(Haulage4.getName());

    tft.setCursor(15, 120);
    tft.println(Haulage5.getName());

    tft.setCursor(15, 140);
    tft.println(Haulage6.getName());

    tft.setCursor(15, 160);
    tft.println(Haulage7.getName());

    tft.setCursor(15, 180);
    tft.println(Haulage8.getName());

    tft.setCursor(15, 200);
    tft.println(Haulage9.getName());

    tft.setCursor(15, 220);
    tft.println(Haulage10.getName());
}

void RCTractorControllerV2::MenuBackground4() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Construction1.getName());

  tft.setCursor(15, 60);
  tft.println(Construction2.getName());

    tft.setCursor(15, 80);
    tft.println(Construction3.getName());

    tft.setCursor(15, 100);
    tft.println(Construction4.getName());

    tft.setCursor(15, 120);
    tft.println(Construction5.getName());

    tft.setCursor(15, 140);
    tft.println(Construction6.getName());

    tft.setCursor(15, 160);
    tft.println(Construction7.getName());

    tft.setCursor(15, 180);
    tft.println(Construction8.getName());

    tft.setCursor(15, 200);
    tft.println(Construction9.getName());

    tft.setCursor(15, 220);
    tft.println(Construction10.getName());
}

void RCTractorControllerV2::MenuBackground5() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Trailer1.getName());

  tft.setCursor(15, 60);
  tft.println(Trailer2.getName());

    tft.setCursor(15, 80);
    tft.println(Trailer3.getName());

    tft.setCursor(15, 100);
    tft.println(Trailer4.getName());

    tft.setCursor(15, 120);
    tft.println(Trailer5.getName());

    tft.setCursor(15, 140);
    tft.println(Trailer6.getName());

    tft.setCursor(15, 160);
    tft.println(Trailer7.getName());

    tft.setCursor(15, 180);
    tft.println(Trailer8.getName());

    tft.setCursor(15, 200);
    tft.println(Trailer9.getName());

    tft.setCursor(15, 220);
    tft.println(Trailer10.getName());
}

void RCTractorControllerV2::MenuBackground6() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(SikuControl1.getName());

  tft.setCursor(15, 60);
  tft.println(SikuControl2.getName());

    tft.setCursor(15, 80);
    tft.println(SikuControl3.getName());

    tft.setCursor(15, 100);
    tft.println(SikuControl4.getName());
}

void RCTractorControllerV2::MenuBackground7() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(IRHeli1.getName());

  tft.setCursor(15, 60);
  tft.println(IRHeli2.getName());

    tft.setCursor(15, 80);
    tft.println(IRHeli3.getName());

    tft.setCursor(15, 100);
    tft.println(IRHeli4.getName());

    tft.setCursor(15, 120);
    tft.println(IRHeli5.getName());

    tft.setCursor(15, 140);
    tft.println(IRHeli6.getName());

    tft.setCursor(15, 160);
    tft.println(IRHeli7.getName());

    tft.setCursor(15, 180);
    tft.println(IRHeli8.getName());

    tft.setCursor(15, 200);
    tft.println(IRHeli9.getName());

    tft.setCursor(15, 220);
    tft.println(IRHeli10.getName());
}

void RCTractorControllerV2::MenuBackground8() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Drone1.getName());

  tft.setCursor(15, 60);
  tft.println(Drone2.getName());

    tft.setCursor(15, 80);
    tft.println(Drone3.getName());

    tft.setCursor(15, 100);
    tft.println(Drone4.getName());

    tft.setCursor(15, 120);
    tft.println(Drone5.getName());

    tft.setCursor(15, 140);
    tft.println(Drone6.getName());

    tft.setCursor(15, 160);
    tft.println(Drone7.getName());

    tft.setCursor(15, 180);
    tft.println(Drone8.getName());

    tft.setCursor(15, 200);
    tft.println(Drone9.getName());

    tft.setCursor(15, 220);
    tft.println(Drone10.getName());
}

void RCTractorControllerV2::MenuBackground9() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(TV1.getName());

  tft.setCursor(15, 60);
  tft.println(TV2.getName());

    tft.setCursor(15, 80);
    tft.println(TV3.getName());

    tft.setCursor(15, 100);
    tft.println(TV4.getName());

    tft.setCursor(15, 120);
    tft.println(TV5.getName());

    tft.setCursor(15, 140);
    tft.println(TV6.getName());

    tft.setCursor(15, 160);
    tft.println(TV7.getName());

    tft.setCursor(15, 180);
    tft.println(TV8.getName());

    tft.setCursor(15, 200);
    tft.println(TV9.getName());

    tft.setCursor(15, 220);
    tft.println(TV10.getName());
}

void RCTractorControllerV2::MenuBackground10() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Game1.getName());

  tft.setCursor(15, 60);
  tft.println(Game2.getName());

    tft.setCursor(15, 80);
    tft.println(Game3.getName());

    tft.setCursor(15, 100);
    tft.println(Game4.getName());

    tft.setCursor(15, 120);
    tft.println(Game5.getName());

    tft.setCursor(15, 140);
    tft.println(Game6.getName());

    tft.setCursor(15, 160);
    tft.println(Game7.getName());

    tft.setCursor(15, 180);
    tft.println(Game8.getName());

    tft.setCursor(15, 200);
    tft.println(Game9.getName());

    tft.setCursor(15, 220);
    tft.println(Game10.getName());
}

void RCTractorControllerV2::MenuBackground11() {
	tft.fillRect(0,0,240,320,BLACK);

  tft.setCursor(15, 40);
  tft.setTextColor(YELLOW);
  tft.setTextSize(2);
  tft.println(Tools1.getName());

  tft.setCursor(15, 60);
  tft.println(Tools2.getName());

    tft.setCursor(15, 80);
    tft.println(Tools3.getName());

    tft.setCursor(15, 100);
    tft.println(Tools4.getName());

    tft.setCursor(15, 120);
    tft.println(Tools5.getName());

    tft.setCursor(15, 140);
    tft.println(Tools6.getName());

    tft.setCursor(15, 160);
    tft.println(Tools7.getName());

    tft.setCursor(15, 180);
    tft.println(Tools8.getName());

    tft.setCursor(15, 200);
    tft.println(Tools9.getName());

    tft.setCursor(15, 220);
    tft.println(Tools10.getName());
}

/*******************************************************************/
/*               Functions to Draw Bitmpas on TFT                  */
/*******************************************************************/

void RCTractorControllerV2::bmpDraw(char *filename, int x, int y) {

  File     bmpFile;
  int      bmpWidth, bmpHeight;   // W+H in pixels
  uint8_t  bmpDepth;              // Bit depth (currently must be 24)
  uint32_t bmpImageoffset;        // Start of image data in file
  uint32_t rowSize;               // Not always = bmpWidth; may have padding
  uint8_t  sdbuffer[3*BUFFPIXEL]; // pixel in buffer (R+G+B per pixel)
  uint16_t lcdbuffer[BUFFPIXEL];  // pixel out buffer (16-bit per pixel)
  uint8_t  buffidx = sizeof(sdbuffer); // Current position in sdbuffer
  boolean  goodBmp = false;       // Set to true on valid header parse
  boolean  flip    = true;        // BMP is stored bottom-to-top
  int      w, h, row, col;
  uint8_t  r, g, b;
  uint32_t pos = 0, startTime = millis();
  uint8_t  lcdidx = 0;
  boolean  first = true;

  if((x >= tft.width()) || (y >= tft.height())) return;

  // Open requested file on SD card
  bmpFile = SD.open(filename);

  // Parse BMP header
  if(read16(bmpFile) == 0x4D42) { // BMP signature
    read32(bmpFile);
    (void)read32(bmpFile); // Read & ignore creator bytes
    bmpImageoffset = read32(bmpFile); // Start of image data
    bmpImageoffset, DEC;
    // Read DIB header
    read32(bmpFile);
    bmpWidth  = read32(bmpFile);
    bmpHeight = read32(bmpFile);
    if(read16(bmpFile) == 1) { // # planes -- must be '1'
      bmpDepth = read16(bmpFile); // bits per pixel
      if((bmpDepth == 24) && (read32(bmpFile) == 0)) { // 0 = uncompressed

        goodBmp = true; // Supported BMP format -- proceed!

        // BMP rows are padded (if needed) to 4-byte boundary
        rowSize = (bmpWidth * 3 + 3) & ~3;

        if(bmpHeight < 0) {
          bmpHeight = -bmpHeight;
          flip      = false;
        }

        // Crop area to be loaded
        w = bmpWidth;
        h = bmpHeight;
        if((x+w-1) >= tft.width())  w = tft.width()  - x;
        if((y+h-1) >= tft.height()) h = tft.height() - y;

        // Set TFT address window to clipped image bounds
        tft.setAddrWindow(x, y, x+w-1, y+h-1);

        for (row=0; row<h; row++) {
         if(flip) // Bitmap is stored bottom-to-top order (normal BMP)
            pos = bmpImageoffset + (bmpHeight - 1 - row) * rowSize;
          else     // Bitmap is stored top-to-bottom
          pos = bmpImageoffset + row * rowSize;
          if(bmpFile.position() != pos) { // Need seek?
            bmpFile.seek(pos);
            buffidx = sizeof(sdbuffer); // Force buffer reload
          }

          for (col=0; col<w; col++) { // For each column...
            // Time to read more pixel data?
            if (buffidx >= sizeof(sdbuffer)) { // Indeed
              // Push LCD buffer to the display first
              if(lcdidx > 0) {
                tft.pushColors(lcdbuffer, lcdidx, first);
                lcdidx = 0;
                first  = false;
              }
              bmpFile.read(sdbuffer, sizeof(sdbuffer));
              buffidx = 0; // Set index to beginning
            }

            // Convert pixel from BMP to TFT format
            b = sdbuffer[buffidx++];
            g = sdbuffer[buffidx++];
            r = sdbuffer[buffidx++];
            lcdbuffer[lcdidx++] = tft.color565(r,g,b);
          } // end pixel
        } // end scanline
        // Write any remaining data to LCD
        if(lcdidx > 0) {
          tft.pushColors(lcdbuffer, lcdidx, first);
        }
      } // end goodBmp
    }
  }

  bmpFile.close();
}

uint16_t RCTractorControllerV2::read16(File f) {
  uint16_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read(); // MSB
  return result;
}

uint32_t RCTractorControllerV2::read32(File f) {
  uint32_t result;
  ((uint8_t *)&result)[0] = f.read(); // LSB
  ((uint8_t *)&result)[1] = f.read();
  ((uint8_t *)&result)[2] = f.read();
  ((uint8_t *)&result)[3] = f.read(); // MSB
  return result;
}

/*******************************************************************/
/*           Functions used to Control RC Vehicles                 */
/*******************************************************************/

void RCTractorControllerV2::Start_Hitachi(){
	int arm1_pos = 127;
	int arm2_pos = 127;
	int slew_pos = 127;

	int arm1_timer = millis();
	int arm1_space = 1000;
	int arm1_dir = 0;

	while(Vehicle_Active != 0){
   		Check_Joys();
		Check_Encoders();

		if(command[3] < 125){
			arm1_space = map(command[3],125,0,1000,20);
    		arm1_dir = 1;
		}
		else if(command[3]>130){
			arm1_space = map(command[3],130,255,1000,20);
    		arm1_dir = 2;
		}
		else{arm1_dir = 0;}

		if((millis()-arm1_timer)>arm1_space){
		    if (arm1_pos != 0x00 && arm1_dir == 1){
		      arm1_pos --;
		        delay(5);
		      Serial.write(0x01);  // Command to enter steering mode
		      delay(5);
		      Serial.write(arm1_pos);  // Steering value
		    }
		    else if (arm1_pos != 0xFF && arm1_dir == 2){
		      arm1_pos ++;
		         delay(5);
		      Serial.write(0x01);  // Command to enter steering mode
		      delay(5);
		      Serial.write(arm1_pos);  // Steering value
		    }
		    arm1_timer = millis();
	   }

/*
		  if(command[2] < 100){
		    if (arm2_pos != 0x00){
		      arm2_pos = arm2_pos - 3;
		       delay(5);
		      Serial.write(0x02);  // Command to enter drive mode
		      delay(5);
		      Serial.write(arm2_pos);  // Drive value
		    }
		  }
		  else if(command[2]>150){
		    if (arm2_pos != 0xFF){
		      arm2_pos = arm2_pos + 3;
		    delay(5);
		      Serial.write(0x02);  // Command to enter drive mode
		      delay(5);
		      Serial.write(arm2_pos);  // Drive value
		    }
		  }


		       delay(5);
		      Serial.write(0x03);  // Command to enter drive mode
		      delay(5);
		      Serial.write(command[4]);  // Drive value



		  if(command[1] < 100){
		    if (slew_pos != 0x00){
		      slew_pos = slew_pos -2;
		      delay(5);
		      Serial.write(0x04);  // Command to enter drive mode
		      delay(5);
		      Serial.write(slew_pos);  // Drive value
		    }
		  }
		  else if(command[1]>150){
		    if (slew_pos != 0xFF){
		      slew_pos = slew_pos +2;
		    delay(5);
		      Serial.write(0x04);  // Command to enter drive mode
		      delay(5);
		      Serial.write(slew_pos);  // Drive value
		    }
		  }


		    delay(5);
		  Serial.write(0x06);  // Command to enter drive mode
		  delay(5);
		  Serial.write(command[5]);  // Drive value

		    delay(5);
		  Serial.write(0x05);  // Command to enter drive mode
		  delay(5);
  Serial.write(command[7]);  // Drive value

		if ((digitalRead(_S_Back))==LOW)
		{
			Vehicle_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

			// Wait for switch to be released
			while ((digitalRead(_S_Back)) == LOW){}
    	}*/
	}
}

void RCTractorControllerV2::Start_Vehicle(){

	command[0]=Vehicle_Active;
	Vehicle_Gear = 3;
	Lost_Packet_Count = 0;
	Reception_Meter = 0;
	Battery_Meter = 0;
	Draw_Dash();
	tft.setTextSize(2);
	tft.setCursor(5, 5);
	tft.setTextColor(YELLOW);
	tft.print("ID:");
	tft.print(command[0]);
	tft.setCursor(100, 5);
	if(Com_Method == 0){tft.print("NRF");}
	if(Com_Method == 1){tft.print("XBee");}
	if(Com_Method == 2){tft.print("IR32");}
	tft.setCursor(20, 300);
	tft.print(Current_Vehicle_Name);
	Draw_Controller_Details();
	Controller_V_Measure_Period=50;
	Vehicle_V_Measure_Period=25;
	unsigned long time = micros();
	Send_Timer = millis();

	while(Vehicle_Active != 0){

		if((micros() - time)>500000){
			Indicator_Symbol();
			time = micros();
		}

		//Check_Fast_Select();
		Check_Light_Buttons();
		Check_Func_Buttons();
		Light_Symbol();

		Controller_Voltage();

		// Tractor_Voltage();

		Update_Dash(0);

   		Check_Joys();
		Check_Encoders();

  		Vehicle_Voltage(Vehicle_V);

		Update_Dash(1);

		if ((digitalRead(_S_Up))==LOW)
		{
			if(Vehicle_Gear != 3){

				Vehicle_Gear++;

				if(Vehicle_Gear == 3){
					tft.setTextSize(2);
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(BLACK);
  					tft.println("L2");
					tft.setCursor((Dashx-4), (Dashy+30));
					tft.setTextColor(YELLOW);
  					tft.println("H");
				}
				else if(Vehicle_Gear == 2){
					tft.setTextSize(2);
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(BLACK);
  					tft.println("L1");
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(YELLOW);
  					tft.println("L2");
				}

			}

			// Wait for switch to be released
			while ((digitalRead(_S_Up)) == LOW){}
    	}

		if ((digitalRead(_S_Down))==LOW)
		{
			if(Vehicle_Gear != 1){

				Vehicle_Gear--;

				if(Vehicle_Gear == 2){
					tft.setTextSize(2);
					tft.setCursor((Dashx-4), (Dashy+30));
					tft.setTextColor(BLACK);
  					tft.println("H");
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(YELLOW);
  					tft.println("L2");
				}
				else if(Vehicle_Gear == 1){
					tft.setTextSize(2);
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(BLACK);
  					tft.println("L2");
					tft.setCursor((Dashx-9), (Dashy+30));
					tft.setTextColor(YELLOW);
  					tft.println("L1");
				}
			}

			// Wait for switch to be released
			while ((digitalRead(_S_Down)) == LOW){}
    	}

		if(Com_Method == 0){Send_NRF();}
		if(Com_Method == 1){Send_NRF();}
		if(Com_Method == 2){Send_IR32();}

		Update_Controller_Details();

		if ((digitalRead(_S_Back))==LOW)
		{
			Vehicle_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

			// Wait for switch to be released
			while ((digitalRead(_S_Back)) == LOW){}
    	}
	}
}

void RCTractorControllerV2::Send_NRF(){
  		// First, stop listening so we can talk.
    	radio.stopListening();

  		// Take the time, and send it.  This will block until complete
		unsigned long time = millis();
  		radio.write( command, sizeof(command));

		// Now, continue listening
    	radio.startListening();

    // Wait here until we get a response, or timeout (250ms)
    unsigned long started_waiting_at = millis();
    bool timeout = false;
    while ( ! radio.available() && ! timeout )
      if (millis() - started_waiting_at > 20 )
        timeout = true;

    // Describe the results
    if ( timeout )
    {
		Lost_Packet_Count++;
    }
    else
    {
      // Grab the response, compare, and send to debugging spew
      radio.read( &Vehicle_V, sizeof(&Vehicle_V) );
      Lost_Packet_Count = 0;
    }
}

void RCTractorControllerV2::Send_XBee(){

}

void RCTractorControllerV2::Send_IR32(){


	if(Vehicle_Active == 31){
		IR32_BYTE1 = 0x80;
		Com_Method_Timer = 64.8;
		}
	else if(Vehicle_Active == 32){
		IR32_BYTE1 = 0x40;
		Com_Method_Timer = 73.8;
		}
	else if(Vehicle_Active == 33){
		IR32_BYTE1 = 0x20;
		Com_Method_Timer = 82.8;
		}
	else if(Vehicle_Active == 34){
		IR32_BYTE1 = 0x10;
		Com_Method_Timer = 91.8;
		}
	if((millis()-Send_Timer)>Com_Method_Timer){

		if ((bitRead(command[9], 2))!=1){bitClear(IR32_BYTE1,0);}else{bitSet(IR32_BYTE1,0);};
		if ((bitRead(command[9], 4))!=1){bitClear(IR32_BYTE1,1);}else{bitSet(IR32_BYTE1,1);};
		if ((bitRead(command[10], 1))!=1){bitClear(IR32_BYTE1,2);}else{bitSet(IR32_BYTE1,2);};
		if ((bitRead(command[10], 2))!=1){bitClear(IR32_BYTE1,3);}else{bitSet(IR32_BYTE1,3);};

		byte IR32_Temp1 = bitswap(IR32_BYTE1);
		byte IR32_Temp2 = 0;
		if(command[2]<=120){IR32_Temp2 = map(command[2],120,0,1,90);}
		else if(command[2]>=135){IR32_Temp2 = map(command[2],135,255,128,218);}
		else{IR32_Temp2 = 0;}
		byte IR32_Temp3 = map(command[3],0,255,50,205);
		byte IR32_Temp4 = command[5];
		IR32_Temp4 = IR32_Temp4 >> 1;
		IR32_Temp4 = map(IR32_Temp4,0,128,30,80);
		byte IR32_Temp5 = command[6];
		IR32_Temp5 = IR32_Temp5 >> 1;

		if ((bitRead(command[9], 0))!=1){bitClear(IR32_Temp5,7);}else{bitSet(IR32_Temp5,7);};
		if ((bitRead(command[9], 1))!=1){bitClear(IR32_Temp4,7);}else{bitSet(IR32_Temp4,7);};

		byte IR32_Temp6 = command[7];

		byte IR32_BYTE2 = bitswap(IR32_Temp2);
		byte IR32_BYTE3 = bitswap(IR32_Temp3);
		byte IR32_BYTE4 = bitswap(IR32_Temp4);
		byte IR32_BYTE5 = bitswap(IR32_Temp5);
		byte IR32_BYTE6 = bitswap(IR32_Temp6);

		byte IR32_TempResult = IR32_Temp1 + IR32_Temp2;
		IR32_TempResult = IR32_TempResult + IR32_Temp3;
		IR32_TempResult = IR32_TempResult + IR32_Temp4;
		IR32_TempResult = IR32_TempResult + IR32_Temp5;
		IR32_TempResult = IR32_TempResult + IR32_Temp6;

		byte IR32_Checksum = bitswap(IR32_TempResult);

		Control32_Pulse(20);
		if(bitRead(IR32_BYTE1,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE1,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_BYTE2,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE2,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_BYTE3,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE3,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_BYTE4,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE4,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_BYTE5,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE5,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_BYTE6,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_BYTE6,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		if(bitRead(IR32_Checksum,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
		if(bitRead(IR32_Checksum,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

		Send_Timer = millis();
	}
}

void RCTractorControllerV2::Draw_Dash(){
	tft.fillRect(0,0,240,320,BLACK);


	double pi = 3.14159265;
	double x = 0; // setting variable to 0 is not needed, it will be written over anyways, it just looks good.
	double y = 0;// same as above

// Draw Odometer
	for (int rad = 55; rad <= 56; rad += 1) {
		for (double i = -(pi+(pi/3)); i <=  (pi/3); i += .01) {
	  		x = Dashx - sin(i + (pi/2)) * rad;
	  		y = Dashy - cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y,WHITE);
		}
	}

// Draw Tachometer
	for (int rad = 55; rad <= 56; rad += 1) {
		for (double i = -(pi/1.5); i <=  (pi/3); i += .01) {
	  		x = (Dashx-75) - sin(i + (pi/2)) * rad;
	  		y = (Dashy) - cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y,WHITE);
		}
	}

// Draw Volt Meter
	for (int rad = 20; rad <= 21; rad += 1) {
		for (double i = -(pi/3); i <=  (pi/3); i += .01) {
	  		x = (Dashx-150) - sin(i + (pi/2)) * rad;
	  		y = (Dashy-25) - cos(i + (pi/2)) * rad;
	  		if(i <=  (pi/6)){
	  			tft.drawPixel(x, y,GREEN);
			}
			else{
				tft.drawPixel(x, y,RED);
			}
		}
	}

// Draw Front Link Position
	for (int rad = 20; rad <= 21; rad += 1) {
		for (double i = -(pi/3); i <=  (pi/3); i += .01) {
	  		x = (Dashx-136) - sin(i + (pi/2)) * rad;
	  		y = (Dashy+80) - cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y, WHITE);
		}
	}

// Draw Rear Link Position
	for (int rad = 20; rad <= 21; rad += 1) {
		for (double i = -(pi/3); i <=  (pi/3); i += .01) {
	  		x = (Dashx-86) - sin(i + (pi/2)) * rad;
	  		y = (Dashy+80) - cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y, WHITE);
		}
	}

// Draw External 1 Position
	for (int rad = 20; rad <= 21; rad += 1) {
		for (double i = -(pi/3); i <=  (pi/3); i += .01) {
	  		x = (Dashx-36) + sin(i + (pi/2)) * rad;
	  		y = (Dashy+80) + cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y, WHITE);
		}
	}

// Draw External 2 Position
	for (int rad = 20; rad <= 21; rad += 1) {
		for (double i = -(pi/3); i <=  (pi/3); i += .01) {
	  		x = (Dashx+14) + sin(i + (pi/2)) * rad;
	  		y = (Dashy+80) + cos(i + (pi/2)) * rad;
	  		tft.drawPixel(x, y, WHITE);
		}
	}

	tft.setCursor((Dashx-4), (Dashy+30));
	tft.setTextColor(YELLOW);
	tft.setTextSize(2);
  	tft.println("H");

}

void RCTractorControllerV2::Draw_Controller_Details(){

	tft.fillCircle(Controllerx, Controllery-3, 4, WHITE);

	int Bx = Controllerx+15;
	int By = Controllery-3;

	tft.drawLine(Bx-1,By+5,Bx+10,By+5,WHITE);
	tft.drawLine(Bx+10,By+5,Bx+10,By-13,WHITE);
	tft.drawLine(Bx+10,By-13,Bx+8,By-13,WHITE);
	tft.drawLine(Bx+8,By-13,Bx+8,By-15,WHITE);
	tft.drawLine(Bx+8,By-15,Bx+1,By-15,WHITE);
	tft.drawLine(Bx+1,By-15,Bx+1,By-13,WHITE);
	tft.drawLine(Bx+1,By-13,Bx-1,By-13,WHITE);
	tft.drawLine(Bx-1,By-13,Bx-1,By+5,WHITE);
}

void RCTractorControllerV2::Update_Controller_Details(){
// Indicate Packet Loss
	int LPx = Controllerx;
	int LPy = Controllery;
	double pi = 3.14159265;
	double x = 0; // setting variable to 0 is not needed, it will be written over anyways, it just looks good.
	double y = 0;// same as above

	if(Lost_Packet_Count<40 && 	bitRead(Reception_Meter,4) != 1){
		for (int rad = 19; rad <= 20; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
					tft.drawPixel(x, y, WHITE); // (cx, cy, calculated x, calculated y)
			}
		}
		bitSet(Reception_Meter,4);
	}
	else if(Lost_Packet_Count>40 && 	bitRead(Reception_Meter,4) == 1){
		for (int rad = 19; rad <= 20; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
					tft.drawPixel(x, y, BLACK); // (cx, cy, calculated x, calculated y)
			}
		}
	bitClear(Reception_Meter,4);
	}

	if(Lost_Packet_Count<60 && 	bitRead(Reception_Meter,3) != 1){
		for (int rad = 16; rad <= 17; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
					tft.drawPixel(x, y, WHITE); // (cx, cy, calculated x, calculated y)
			}
		}
		bitSet(Reception_Meter,3);
	}
	else if(Lost_Packet_Count>60 && 	bitRead(Reception_Meter,3) == 1){
		for (int rad = 16; rad <= 17; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
					tft.drawPixel(x, y, BLACK); // (cx, cy, calculated x, calculated y)
			}
		}
		bitClear(Reception_Meter,3);
	}

	if(Lost_Packet_Count<80 && 	bitRead(Reception_Meter,2) != 1){
		for (int rad = 13; rad <= 14; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
				tft.drawPixel(x, y, WHITE); // (cx, cy, calculated x, calculated y)
			}
		}
		bitSet(Reception_Meter,2);
	}
	else if(Lost_Packet_Count>80 && 	bitRead(Reception_Meter,2) == 1){
		for (int rad = 13; rad <= 14; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
				tft.drawPixel(x, y, BLACK); // (cx, cy, calculated x, calculated y)
			}
		}
		bitClear(Reception_Meter,2);
	}

	if(Lost_Packet_Count<100 && 	bitRead(Reception_Meter,1) != 1){
		for (int rad = 10; rad <= 11; rad += 1) {
			for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
				x = LPx - sin(i + (pi/2)) * rad;
				y = LPy - cos(i + (pi/2)) * rad;
					tft.drawPixel(x, y, WHITE); // (cx, cy, calculated x, calculated y)
			}
		}
		bitSet(Reception_Meter,1);
	}
	else if(Lost_Packet_Count>100 && 	bitRead(Reception_Meter,1) == 1){
			for (int rad = 10; rad <= 11; rad += 1) {
				for (double i = -(pi/3); i >=  -((2*pi)/3); i -= .01) {
					x = LPx - sin(i + (pi/2)) * rad;
					y = LPy - cos(i + (pi/2)) * rad;
						tft.drawPixel(x, y, BLACK); // (cx, cy, calculated x, calculated y)
				}
			}
				bitClear(Reception_Meter,1);
	}

// Indicate Battery Level
	int Bx = Controllerx+15;
	int By = Controllery-3;

	if(Controller_V_Average>8 && 	bitRead(Battery_Meter,3) != 1){
		tft.fillRect(Bx,By-12,10,5,GREEN);
		tft.fillRect(Bx+2,By-14,6,2,GREEN);
		bitSet(Battery_Meter,3);
	}
	else if(Controller_V_Average<8 && 	bitRead(Battery_Meter,3) == 1){
		tft.fillRect(Bx,By-12,10,5,BLACK);
		tft.fillRect(Bx+2,By-14,6,2,BLACK);
		bitClear(Battery_Meter,3);
	}

	if(Controller_V_Average>7 && 	bitRead(Battery_Meter,2) != 1){
		tft.fillRect(Bx,By-6,10,5,GREEN);
		bitSet(Battery_Meter,2);
	}
	else if(Controller_V_Average<7 && 	bitRead(Battery_Meter,2) == 1){
		tft.fillRect(Bx,By-6,10,5,BLACK);
		bitClear(Battery_Meter,2);
	}

	if(Controller_V_Average>6 && 	bitRead(Battery_Meter,1) != 1){
		tft.fillRect(Bx,By,10,5,RED);
		bitSet(Battery_Meter,1);
	}
	else if(Controller_V_Average<5 && 	bitRead(Battery_Meter,1) == 1){
		tft.fillRect(Bx,By,10,6,BLACK);
		bitClear(Battery_Meter,1);
	}

}

void RCTractorControllerV2::Update_Dash(int clear){

	float i = 0.000000;
	int rad = 0;
	double pi = 3.14159265;
	double x = 0; // setting variable to 0 is not needed, it will be written over anyways, it just looks good.
	double y = 0;// same as above
	int Throttle_Pos = 0;
	double Link_Pos = 0;

	if(clear==0){
		// Draw Odometer Needle
		if(command[2]>127){
			Throttle_Pos = map(command[2], 127, 255, 0, 255);
		}
		else{
			Throttle_Pos = map(command[2], 127, 0, 0, 255);
		}
		i = (pi/3)-(((((500/3)*pi)/255)*Throttle_Pos)/100);
		rad = 54;
		x = Dashx - sin(i + (pi/2)) * rad;
		y = Dashy - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx, Dashy, x, y, BLACK);


		// Draw Tachometer Needle
		if(Throttle_Position>127){
			Throttle_Pos = map(Throttle_Position, 127, 255, 0, 255);
		}
		else{
			Throttle_Pos = map(Throttle_Position, 127, 0, 0, 255);
		}
		i = (pi/3)-((pi/255)*Throttle_Pos);
		rad = 54;
		x = (Dashx-75) - sin(i + (pi/2)) * rad;
		y = (Dashy) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-75, Dashy, x, y, BLACK);

		// Draw Volt Meter Needle
		Vehicle_V_Average = constrain(Vehicle_V_Average, 3.6, 4.2);
		//if(Vehicle_V_Average<3.4){Vehicle_V_Average=3.4;}
		//if(Vehicle_V_Average>4.2){Vehicle_V_Average=4.2;}
		i = (pi/3)-((((2*pi)/3)/0.6)*(Vehicle_V_Average-3.6));
		rad = 19;
		x = (Dashx-150) - sin(i + (pi/2)) * rad;
		y = (Dashy-25) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-150, Dashy-25, x, y, BLACK);

		// Draw Front Link Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[6])/100);
		rad = 19;
		x = (Dashx-136) - sin(i + (pi/2)) * rad;
		y = (Dashy+80) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-136, Dashy+80, x, y, BLACK);

		// Draw Rear Link Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[5])/100);
		rad = 19;
		x = (Dashx-86) - sin(i + (pi/2)) * rad;
		y = (Dashy+80) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-86, Dashy+80, x, y, BLACK);

		// Draw Function 1 Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[7])/100);
		rad = 19;
		x = (Dashx-36) + sin(i + (pi/2)) * rad;
		y = (Dashy+80) + cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-36, Dashy+80, x, y, BLACK);

		// Draw Function 2 Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[8])/100);
		rad = 19;
		x = (Dashx+14) + sin(i + (pi/2)) * rad;
		y = (Dashy+80) + cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx+14, Dashy+80, x, y, BLACK);


	}
	else{

		// Draw Odometer Needle
		if(command[2]>127){
			Throttle_Pos = map(command[2], 127, 255, 0, 255);
		}
		else{
			Throttle_Pos = map(command[2], 127, 0, 0, 255);
		}
		i = (pi/3)-(((((500/3)*pi)/255)*Throttle_Pos)/100);
		rad = 54;
		x = Dashx - sin(i + (pi/2)) * rad;
		y = Dashy - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx, Dashy, x, y, RED);


		// Draw Tachometer Needle
		if(Throttle_Position>127){
			Throttle_Pos = map(Throttle_Position, 127, 255, 0, 255);
		}
		else{
			Throttle_Pos = map(Throttle_Position, 127, 0, 0, 255);
		}
		i = (pi/3)-((pi/255)*Throttle_Pos);
		rad = 54;
		x = (Dashx-75) - sin(i + (pi/2)) * rad;
		y = (Dashy) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-75, Dashy, x, y, RED);

		// Draw Volt Meter Needle
		Vehicle_V_Average = constrain(Vehicle_V_Average, 3.6, 4.2);
		//if(Vehicle_V_Average<3.4){Vehicle_V_Average=3.4;}
		//if(Vehicle_V_Average>4.2){Vehicle_V_Average=4.2;}
		i = (pi/3)-((((2*pi)/3)/0.6)*(Vehicle_V_Average-3.6));
		rad = 19;
		x = (Dashx-150) - sin(i + (pi/2)) * rad;
		y = (Dashy-25) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-150, Dashy-25, x, y, RED);


		// Draw Front Link Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[6])/100);
		rad = 19;
		x = (Dashx-136) - sin(i + (pi/2)) * rad;
		y = (Dashy+80) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-136, Dashy+80, x, y, RED);

		// Draw Rear Link Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[5])/100);
		rad = 19;
		x = (Dashx-86) - sin(i + (pi/2)) * rad;
		y = (Dashy+80) - cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-86, Dashy+80, x, y, RED);

		// Draw Function 1 Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[7])/100);
		rad = 19;
		x = (Dashx-36) + sin(i + (pi/2)) * rad;
		y = (Dashy+80) + cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx-36, Dashy+80, x, y, RED);

		// Draw Function 2 Needle
		i = (pi/3)-(((((200/3)*pi)/255)*command[8])/100);
		rad = 19;
		x = (Dashx+14) + sin(i + (pi/2)) * rad;
		y = (Dashy+80) + cos(i + (pi/2)) * rad;
		tft.drawLine(Dashx+14, Dashy+80, x, y, RED);
	}
}

void RCTractorControllerV2::Indicator_Symbol(){
  Flash_Timer = Flash_Timer+1;
  int lx = (Dashx-180+5);
  int ly = (Dashy-70);
  int rx = (Dashx+59-5);
  int ry = (Dashy-70);

  if(bitRead(command[9],1)==1 && bitRead(command[9],0)==0 && Flash_Timer == 1){
    tft.fillRect(lx+15,ly-5,20,10,GREEN);
    tft.fillTriangle(lx,ly,lx+15,ly-10,lx+15,ly+10,GREEN);
  }
  else if(bitRead(command[9],1)==0 && bitRead(command[9],0)==1 && Flash_Timer == 1){
    tft.fillRect(rx-35,ry-5,20,10,GREEN);
    tft.fillTriangle(rx,ry,rx-15,ry-10,rx-15,ry+10,GREEN);
  }
  else if(bitRead(command[9],0)==1 && bitRead(command[9],1)==1 && Flash_Timer == 1){
    tft.fillRect(rx-35,ry-5,20,10,GREEN);
    tft.fillTriangle(rx,ry,rx-15,ry-10,rx-15,ry+10,GREEN);

    tft.fillRect(lx+15,ly-5,20,10,GREEN);
    tft.fillTriangle(lx,ly,lx+15,ly-10,lx+15,ly+10,GREEN);
  }

  if(Flash_Timer==2){
    tft.fillRect(rx-35,ry-5,20,10,BLACK);
    tft.fillTriangle(rx,ry,rx-15,ry-10,rx-15,ry+10,BLACK);

    tft.fillRect(lx+15,ly-5,20,10,BLACK);
    tft.fillTriangle(lx,ly,lx+15,ly-10,lx+15,ly+10,BLACK);
  	Flash_Timer=0;
  }
}

void RCTractorControllerV2::Light_Symbol(){
  int dx = (Dashx-180+57);
  int dy = (Dashy-105);
  int mx = (Dashx+59-123);
  int my = (Dashy-105);
  int wx = (Dashx+59-63);
  int wy = (Dashy-105);

  if(bitRead(command[9],3)==1 && bitRead(Current_Light_Symbol,1)!=1){
    tft.fillRect(mx-18,my+1,39,20,BLACK);
    tft.setCursor(mx, my);
    tft.setTextColor(BLUE);
    tft.setTextSize(3);
    tft.println("C");

    tft.drawPixel(mx+1,my+2,BLUE);
    tft.drawPixel(mx+2,my+2,BLUE);
    tft.drawPixel(mx+2,my+1,BLUE);

    tft.drawPixel(mx+1,my+18,BLUE);
    tft.drawPixel(mx+2,my+18,BLUE);
    tft.drawPixel(mx+2,my+19,BLUE);

    tft.drawPixel(mx+12,my+4,BLACK);
    tft.drawPixel(mx+13,my+4,BLACK);
    tft.drawPixel(mx+13,my+5,BLACK);
    tft.drawPixel(mx+12,my+5,BLACK);
    tft.drawPixel(mx+13,my+6,BLACK);
    tft.drawPixel(mx+14,my+5,BLACK);
    tft.drawPixel(mx+14,my+6,BLACK);

    tft.drawPixel(mx+12,my+16,BLACK);
    tft.drawPixel(mx+13,my+16,BLACK);
    tft.drawPixel(mx+13,my+15,BLACK);
    tft.drawPixel(mx+12,my+15,BLACK);
    tft.drawPixel(mx+13,my+14,BLACK);
    tft.drawPixel(mx+14,my+15,BLACK);
    tft.drawPixel(mx+14,my+14,BLACK);

    tft.fillRect(mx-18,my+3,15,2,BLUE);
    tft.fillRect(mx-18,my+7,15,2,BLUE);
    tft.fillRect(mx-18,my+11,15,2,BLUE);
    tft.fillRect(mx-18,my+15,15,2,BLUE);

    tft.fillRect(mx+10,my+1,4,2,BLUE);
    tft.fillRect(mx+12,my+2,4,2,BLUE);
    tft.fillRect(mx+14,my+3,4,2,BLUE);
    tft.fillRect(mx+16,my+4,4,2,BLUE);
    tft.fillRect(mx+17,my+5,4,2,BLUE);
    tft.fillRect(mx+18,my+6,4,2,BLUE);
    tft.fillRect(mx+19,my+7,3,2,BLUE);
    tft.fillRect(mx+20,my+8,3,5,BLUE);
    tft.fillRect(mx+19,my+12,3,2,BLUE);
    tft.fillRect(mx+18,my+13,4,2,BLUE);
    tft.fillRect(mx+17,my+14,4,2,BLUE);
    tft.fillRect(mx+16,my+15,4,2,BLUE);
    tft.fillRect(mx+14,my+16,4,2,BLUE);
    tft.fillRect(mx+12,my+17,4,2,BLUE);
    tft.fillRect(mx+10,my+18,4,2,BLUE);
    bitSet(Current_Light_Symbol,1);
  }

  if(bitRead(command[9],2)==1 && bitRead(Current_Light_Symbol,2)!=1){
    tft.fillRect(dx-18,dy+1,39,20,BLACK);
    tft.setCursor(dx, dy);
    tft.setTextColor(GREEN);
    tft.setTextSize(3);
    tft.println("C");

    tft.drawPixel(dx+1,dy+2,GREEN);
    tft.drawPixel(dx+2,dy+2,GREEN);
    tft.drawPixel(dx+2,dy+1,GREEN);

    tft.drawPixel(dx+1,dy+18,GREEN);
    tft.drawPixel(dx+2,dy+18,GREEN);
    tft.drawPixel(dx+2,dy+19,GREEN);

    tft.drawPixel(dx+12,dy+4,BLACK);
    tft.drawPixel(dx+13,dy+4,BLACK);
    tft.drawPixel(dx+13,dy+5,BLACK);
    tft.drawPixel(dx+12,dy+5,BLACK);
    tft.drawPixel(dx+13,dy+6,BLACK);
    tft.drawPixel(dx+14,dy+5,BLACK);
    tft.drawPixel(dx+14,dy+6,BLACK);

    tft.drawPixel(dx+12,dy+16,BLACK);
    tft.drawPixel(dx+13,dy+16,BLACK);
    tft.drawPixel(dx+13,dy+15,BLACK);
    tft.drawPixel(dx+12,dy+15,BLACK);
    tft.drawPixel(dx+13,dy+14,BLACK);
    tft.drawPixel(dx+14,dy+15,BLACK);
    tft.drawPixel(dx+14,dy+14,BLACK);

    tft.drawLine(dx-3,dy+3,dx-18,dy+7,GREEN);
    tft.drawLine(dx-3,dy+4,dx-18,dy+8,GREEN);

    tft.drawLine(dx-3,dy+7,dx-18,dy+11,GREEN);
    tft.drawLine(dx-3,dy+8,dx-18,dy+12,GREEN);

    tft.drawLine(dx-3,dy+11,dx-18,dy+15,GREEN);
    tft.drawLine(dx-3,dy+12,dx-18,dy+16,GREEN);

    tft.drawLine(dx-3,dy+15,dx-18,dy+19,GREEN);
    tft.drawLine(dx-3,dy+16,dx-18,dy+20,GREEN);

    tft.fillRect(dx+10,dy+1,4,2,GREEN);
    tft.fillRect(dx+12,dy+2,4,2,GREEN);
    tft.fillRect(dx+14,dy+3,4,2,GREEN);
    tft.fillRect(dx+16,dy+4,4,2,GREEN);
    tft.fillRect(dx+17,dy+5,4,2,GREEN);
    tft.fillRect(dx+18,dy+6,4,2,GREEN);
    tft.fillRect(dx+19,dy+7,3,2,GREEN);
    tft.fillRect(dx+20,dy+8,3,5,GREEN);
    tft.fillRect(dx+19,dy+12,3,2,GREEN);
    tft.fillRect(dx+18,dy+13,4,2,GREEN);
    tft.fillRect(dx+17,dy+14,4,2,GREEN);
    tft.fillRect(dx+16,dy+15,4,2,GREEN);
    tft.fillRect(dx+14,dy+16,4,2,GREEN);
    tft.fillRect(dx+12,dy+17,4,2,GREEN);
    tft.fillRect(dx+10,dy+18,4,2,GREEN);
    bitSet(Current_Light_Symbol,2);
  }

  if(bitRead(command[9],4)==1 && bitRead(Current_Light_Symbol,3)!=1){
    tft.setCursor(wx, wy);
    tft.setTextColor(ORANGE);
    tft.setTextSize(3);
    tft.println("C");

    tft.drawPixel(wx+1,wy+2,ORANGE);
    tft.drawPixel(wx+2,wy+2,ORANGE);
    tft.drawPixel(wx+2,wy+1,ORANGE);

    tft.drawPixel(wx+1,wy+18,ORANGE);
    tft.drawPixel(wx+2,wy+18,ORANGE);
    tft.drawPixel(wx+2,wy+19,ORANGE);

    tft.drawPixel(wx+12,wy+3,BLACK);
    tft.drawPixel(wx+13,wy+4,BLACK);
    tft.drawPixel(wx+13,wy+5,BLACK);
    tft.drawPixel(wx+12,wy+5,BLACK);
    tft.drawPixel(wx+13,wy+6,BLACK);
    tft.drawPixel(wx+14,wy+5,BLACK);
    tft.drawPixel(wx+14,wy+6,BLACK);

    tft.drawPixel(wx+12,wy+16,BLACK);
    tft.drawPixel(wx+13,wy+16,BLACK);
    tft.drawPixel(wx+13,wy+15,BLACK);
    tft.drawPixel(wx+12,wy+15,BLACK);
    tft.drawPixel(wx+13,wy+14,BLACK);
    tft.drawPixel(wx+14,wy+15,BLACK);
    tft.drawPixel(wx+14,wy+14,BLACK);

    tft.drawLine(wx-3,wy+3,wx-18,wy,ORANGE);
    tft.drawLine(wx-3,wy+4,wx-18,wy+1,ORANGE);

    tft.drawLine(wx-3,wy+7,wx-18,wy+5,ORANGE);
    tft.drawLine(wx-3,wy+8,wx-18,wy+6,ORANGE);

    tft.drawLine(wx-3,wy+11,wx-18,wy+13,ORANGE);
    tft.drawLine(wx-3,wy+12,wx-18,wy+14,ORANGE);

    tft.drawLine(wx-3,wy+15,wx-18,wy+19,ORANGE);
    tft.drawLine(wx-3,wy+16,wx-18,wy+20,ORANGE);

    tft.fillRect(wx+10,wy+1,4,2,ORANGE);
    tft.fillRect(wx+12,wy+2,4,2,ORANGE);
    tft.fillRect(wx+14,wy+3,4,2,ORANGE);
    tft.fillRect(wx+16,wy+4,4,2,ORANGE);
    tft.fillRect(wx+17,wy+5,4,2,ORANGE);
    tft.fillRect(wx+18,wy+6,4,2,ORANGE);
    tft.fillRect(wx+19,wy+7,3,2,ORANGE);
    tft.fillRect(wx+20,wy+8,3,5,ORANGE);
    tft.fillRect(wx+19,wy+12,3,2,ORANGE);
    tft.fillRect(wx+18,wy+13,4,2,ORANGE);
    tft.fillRect(wx+17,wy+14,4,2,ORANGE);
    tft.fillRect(wx+16,wy+15,4,2,ORANGE);
    tft.fillRect(wx+14,wy+16,4,2,ORANGE);
    tft.fillRect(wx+12,wy+17,4,2,ORANGE);
    tft.fillRect(wx+10,wy+18,4,2,ORANGE);
    bitSet(Current_Light_Symbol,3);
  }

if(bitRead(Current_Light_Symbol,1)!=0 || bitRead(Current_Light_Symbol,2)!=0 || bitRead(Current_Light_Symbol,3)!=0){
    if(bitRead(command[9],3)==0){
    tft.fillRect(mx-18,my,42,22,BLACK);
    bitClear(Current_Light_Symbol,1);
  }
if(bitRead(command[9],2)==0){
    tft.fillRect(dx-18,dy,42,22,BLACK);
    bitClear(Current_Light_Symbol,2);
  }
if(bitRead(command[9],4)==0){
    tft.fillRect(wx-18,wy,42,22,BLACK);
    bitClear(Current_Light_Symbol,3);
  }
}
}

/*******************************************************************/
/*       Functions used to Control Siku Control 32 Tractors        */
/*******************************************************************/

void RCTractorControllerV2::Start_Control32(){
	tft.fillRect(0,0,240,320,BLACK);
	tft.setTextColor(GREEN);
				tft.setTextSize(2);
				tft.setCursor(5, 5);
			tft.println("Control 32");
			digitalWrite(47,HIGH);

	switch (Control32_Active) {
		case 1:{
			while(Control32_Active!=0){
				Control32_Pulse(20);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);
				Control32_Pulse(15);
				Control32_Pulse(25);
				Control32_Pulse(15);


				delay(62);

				if ((digitalRead(_S_Back))==LOW)
				{
					Control32_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();


					// Wait for switch to be released
					while ((digitalRead(_S_Back)) == LOW){}
				}
			}
			break;
		}

	case 2:{
				while(Control32_Active!=0){

					Check_Joys();
					Check_Encoders();
					Check_Light_Buttons();
					Check_Func_Buttons();


								byte IR32_BYTE1 = 0x80;
								if ((bitRead(command[9], 2))!=1){bitClear(IR32_BYTE1,0);}else{bitSet(IR32_BYTE1,0);};
								if ((bitRead(command[10], 0))!=1){bitClear(IR32_BYTE1,1);}else{bitSet(IR32_BYTE1,1);};
								if ((bitRead(command[10], 1))!=1){bitClear(IR32_BYTE1,2);}else{bitSet(IR32_BYTE1,2);};
								if ((bitRead(command[10], 2))!=1){bitClear(IR32_BYTE1,3);}else{bitSet(IR32_BYTE1,3);};

					byte IR32_Temp1 = bitswap(IR32_BYTE1);
					byte IR32_Temp2 = 0;
					if(command[2]<=126){IR32_Temp2 = map(command[2],126,0,1,127);}
					else if(command[2]>=128){IR32_Temp2 = command[2];}
					else{IR32_Temp2 = 0;}
					byte IR32_Temp3 = map(command[3],0,255,50,205);
					byte IR32_Temp4 = command[5];
					IR32_Temp4 = IR32_Temp4 >> 1;
					IR32_Temp4 = map(IR32_Temp4,0,128,30,80);
					byte IR32_Temp5 = command[6];
					IR32_Temp5 = IR32_Temp5 >> 1;

								if ((bitRead(command[9], 0))!=1){bitClear(IR32_Temp5,7);}else{bitSet(IR32_Temp5,7);};
								if ((bitRead(command[9], 1))!=1){bitClear(IR32_Temp4,7);}else{bitSet(IR32_Temp4,7);};

					byte IR32_Temp6 = command[7];

											tft.setTextColor(GREEN);
														tft.setTextSize(2);
														tft.setCursor(5, 25);
			tft.println(IR32_Temp3);

								byte IR32_BYTE2 = bitswap(IR32_Temp2);
								byte IR32_BYTE3 = bitswap(IR32_Temp3);
								byte IR32_BYTE4 = bitswap(IR32_Temp4);
								byte IR32_BYTE5 = bitswap(IR32_Temp5);
								byte IR32_BYTE6 = bitswap(IR32_Temp6);

					byte IR32_TempResult = IR32_Temp1 + IR32_Temp2;
					IR32_TempResult = IR32_TempResult + IR32_Temp3;
					IR32_TempResult = IR32_TempResult + IR32_Temp4;
					IR32_TempResult = IR32_TempResult + IR32_Temp5;
					IR32_TempResult = IR32_TempResult + IR32_Temp6;

					byte IR32_Checksum = bitswap(IR32_TempResult);


					Control32_Pulse(20);
					if(bitRead(IR32_BYTE1,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE1,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_BYTE2,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE2,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_BYTE3,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE3,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_BYTE4,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE4,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_BYTE5,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE5,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_BYTE6,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_BYTE6,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					if(bitRead(IR32_Checksum,7)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,6)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,5)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,4)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,3)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,2)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,1)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}
					if(bitRead(IR32_Checksum,0)==1){Control32_Pulse(25);}else{Control32_Pulse(15);}

					delay(62);

						tft.setTextColor(BLACK);
									tft.setTextSize(2);
									tft.setCursor(5, 25);
			tft.println(IR32_Temp3);

					if ((digitalRead(_S_Back))==LOW)
					{
						Control32_Active = 0;
						menu.moveUp();
						Menu_Change = 1;
						menu.moveDown();


						// Wait for switch to be released
						while ((digitalRead(_S_Back)) == LOW){}
					}
				}
				break;
		}
	}

}

unsigned char RCTractorControllerV2::bitswap (unsigned char x)
{
 byte result;

   asm("mov __tmp_reg__, %[in] \n\t"
     "lsl __tmp_reg__  \n\t"   /* shift out high bit to carry */
     "ror %[out] \n\t"  /* rotate carry __tmp_reg__to low bit (eventually) */
     "lsl __tmp_reg__  \n\t"   /* 2 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 3 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 4 */
     "ror %[out] \n\t"

     "lsl __tmp_reg__  \n\t"   /* 5 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 6 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 7 */
     "ror %[out] \n\t"
     "lsl __tmp_reg__  \n\t"   /* 8 */
     "ror %[out] \n\t"
     : [out] "=r" (result) : [in] "r" (x));
     return(result);
}

void RCTractorControllerV2::Control32_Pulse(int Num_Pulse){
	cli();
	while(Num_Pulse!=0){
		PINL = 0x04; // Toggle pin47
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		PINL = 0x04; // Toggle pin 47
		__asm__("nop\n\tnop\n\tnop\n\tnop\n\tnop\n\t");
		__asm__("nop\n\tnop\n\t");
  		__asm__("nop\n\t");
		Num_Pulse --;
	}
	sei();
	delayMicroseconds(80);
}


/*******************************************************************/
/*           Functions used to Control IR Helicopters              */
/*******************************************************************/

void RCTractorControllerV2::Start_IR_Heli(){

	tft.fillRect(0,0,240,320,BLACK);

	// Initialise the encoders
	Encoder1_Limit=127;
	Encoder1.write(127);
	Encoder2.write(28);

	switch (IR_Heli_Active) {
		case 1:{
			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println(IRHeli1.getName());

			tft.setCursor(5, 30);
			tft.println("1. Header 1");
			tft.setCursor(5, 45);
			tft.println("2. Header 2");
			tft.setCursor(5, 60);
			tft.println("3. Gap Pulse");
			tft.setCursor(5, 75);
			tft.println("4. 0 Pulse");
			tft.setCursor(5, 90);
			tft.println("5. 1 Pulse");
			tft.setCursor(5, 105);
			tft.println("6. Pulse Space");

			int IRHeli_1_Pulse = 820;
			int IRHeli_0_Pulse = 400;
			int IRHeli_Gap_Pulse = 360;
			int IRHeli_Header_1 = 400;
			int IRHeli_Header_2 = 1640;
			int IRHeli_Pulse_Space = 136;
			int IRHeli_Menu = 0;


			while(IR_Heli_Active==1){

				Check_Joys();
				Check_Encoders();

				if ((digitalRead(_VQS_V1))==LOW){IRHeli_Menu = 0;while ((digitalRead(_VQS_V1)) == LOW){}}
				if ((digitalRead(_VQS_V2))==LOW){IRHeli_Menu = 1;while ((digitalRead(_VQS_V2)) == LOW){}}
				if ((digitalRead(_VQS_V3))==LOW){IRHeli_Menu = 2;while ((digitalRead(_VQS_V3)) == LOW){}}
				if ((digitalRead(_VQS_V4))==LOW){IRHeli_Menu = 3;while ((digitalRead(_VQS_V4)) == LOW){}}
				if ((digitalRead(_VQS_V5))==LOW){IRHeli_Menu = 4;while ((digitalRead(_VQS_V5)) == LOW){}}
				if ((digitalRead(_VQS_V6))==LOW){IRHeli_Menu = 5;while ((digitalRead(_VQS_V6)) == LOW){}}

switch (IRHeli_Menu) {
		case 0:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 30);
			tft.println(IRHeli_Header_1);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_Header_1 = IRHeli_Header_1+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_Header_1 = IRHeli_Header_1-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 30);
			tft.println(IRHeli_Header_1);
			break;
		}
		case 1:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 45);
			tft.println(IRHeli_Header_2);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_Header_2 = IRHeli_Header_2+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_Header_2 = IRHeli_Header_2-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 45);
			tft.println(IRHeli_Header_2);
			break;
		}
		case 2:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 60);
			tft.println(IRHeli_Gap_Pulse);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_Gap_Pulse = IRHeli_Gap_Pulse+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_Gap_Pulse = IRHeli_Gap_Pulse-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 60);
			tft.println(IRHeli_Gap_Pulse);
			break;
		}
		case 3:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 75);
			tft.println(IRHeli_0_Pulse);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_0_Pulse = IRHeli_0_Pulse+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_0_Pulse = IRHeli_0_Pulse-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 75);
			tft.println(IRHeli_0_Pulse);
			break;
		}
		case 4:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 90);
			tft.println(IRHeli_1_Pulse);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_1_Pulse = IRHeli_1_Pulse+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_1_Pulse = IRHeli_1_Pulse-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 90);
			tft.println(IRHeli_1_Pulse);
			break;
		}
		case 5:{
			tft.setTextColor(BLACK);
			tft.setCursor(180, 105);
			tft.println(IRHeli_Pulse_Space);
  			if ((digitalRead(_S_Up))==LOW)
    		{
				IRHeli_Pulse_Space = IRHeli_Pulse_Space+5;
     	 		while ((digitalRead(_S_Up)) == LOW){}
    		}
    		else if ((digitalRead(_S_Down))==LOW)
			{
				IRHeli_Pulse_Space = IRHeli_Pulse_Space-5;
			     while ((digitalRead(_S_Down)) == LOW){}
    		}
    		tft.setTextColor(GREEN);
			tft.setCursor(180, 105);
			tft.println(IRHeli_Pulse_Space);
			break;
		}
}

			unsigned int RAW_IR[59] = {IRHeli_Header_1, IRHeli_Gap_Pulse, IRHeli_Header_2, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse,
			IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse,
			IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse,
			IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_1_Pulse, IRHeli_Gap_Pulse,
			IRHeli_1_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse, IRHeli_Gap_Pulse, IRHeli_0_Pulse,};

				tft.setTextColor(BLACK);
							tft.setCursor(100, 200);
			tft.println(IRHeli_Throttle);

				IRHeli_Throttle = map(Encoder1NewPosition, 0, Encoder1_Limit, 127, 0);

tft.setTextColor(GREEN);
			tft.setCursor(100, 200);
			tft.println(IRHeli_Throttle);

				IRHeli_Pitch = map(command[3], 0, 255, 63, 0);


				if(command[4]>140){
				IRHeli_Yaw = map(command[4], 140, 255, 9, 15);
				}
				else if(command[4]<114){
					IRHeli_Yaw = map(command[4], 0, 114, 0, 7);
				}
				else{
					IRHeli_Yaw = 8;
				}

				if(Encoder2NewPosition>29){
					IRHeli_Trim = map(Encoder2NewPosition, 29, 51, 29, 63);
				}
				else if(Encoder2NewPosition<22){
					IRHeli_Trim = map(Encoder2NewPosition, 0, 22, 0, 27);
				}
				else{
					IRHeli_Trim = 28;
				}

				// Throttle Data
				if(bitRead(IRHeli_Throttle,6)==0){RAW_IR[4]=IRHeli_0_Pulse;}else{RAW_IR[4]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,5)==0){RAW_IR[6]=IRHeli_0_Pulse;}else{RAW_IR[6]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,4)==0){RAW_IR[8]=IRHeli_0_Pulse;}else{RAW_IR[8]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,3)==0){RAW_IR[10]=IRHeli_0_Pulse;}else{RAW_IR[10]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,2)==0){RAW_IR[12]=IRHeli_0_Pulse;}else{RAW_IR[12]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,1)==0){RAW_IR[14]=IRHeli_0_Pulse;}else{RAW_IR[14]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Throttle,0)==0){RAW_IR[16]=IRHeli_0_Pulse;}else{RAW_IR[16]=IRHeli_1_Pulse;}

				// Pitch Data
				if(bitRead(IRHeli_Pitch,5)==0){RAW_IR[18]=IRHeli_0_Pulse;}else{RAW_IR[18]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Pitch,4)==0){RAW_IR[20]=IRHeli_0_Pulse;}else{RAW_IR[20]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Pitch,3)==0){RAW_IR[22]=IRHeli_0_Pulse;}else{RAW_IR[22]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Pitch,2)==0){RAW_IR[24]=IRHeli_0_Pulse;}else{RAW_IR[24]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Pitch,1)==0){RAW_IR[26]=IRHeli_0_Pulse;}else{RAW_IR[26]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Pitch,0)==0){RAW_IR[28]=IRHeli_0_Pulse;}else{RAW_IR[28]=IRHeli_1_Pulse;}

				// Yaw Data
				if(bitRead(IRHeli_Yaw,3)==0){RAW_IR[38]=IRHeli_0_Pulse;}else{RAW_IR[38]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Yaw,2)==0){RAW_IR[40]=IRHeli_0_Pulse;}else{RAW_IR[40]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Yaw,1)==0){RAW_IR[42]=IRHeli_0_Pulse;}else{RAW_IR[42]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Yaw,0)==0){RAW_IR[44]=IRHeli_0_Pulse;}else{RAW_IR[44]=IRHeli_1_Pulse;}

				// Trim Data
				if(bitRead(IRHeli_Trim,5)==0){RAW_IR[46]=IRHeli_0_Pulse;}else{RAW_IR[46]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Trim,4)==0){RAW_IR[48]=IRHeli_0_Pulse;}else{RAW_IR[48]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Trim,3)==0){RAW_IR[50]=IRHeli_0_Pulse;}else{RAW_IR[50]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Trim,2)==0){RAW_IR[52]=IRHeli_0_Pulse;}else{RAW_IR[52]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Trim,1)==0){RAW_IR[54]=IRHeli_0_Pulse;}else{RAW_IR[54]=IRHeli_1_Pulse;}
				if(bitRead(IRHeli_Trim,0)==0){RAW_IR[56]=IRHeli_0_Pulse;}else{RAW_IR[56]=IRHeli_1_Pulse;}

				//noInterrupts();

				irsend.sendRaw(RAW_IR,59,62); // Sony TV power code

  				//interrupts();

				delay(IRHeli_Pulse_Space);

				if ((digitalRead(_S_Back))==LOW)
				{
					IR_Heli_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

					// Wait for switch to be released
					while ((digitalRead(_S_Back)) == LOW){}
    			}
			}
			break;
		}
		case 2:{
			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println(IRHeli2.getName());

			tft.setCursor(5, 30);
			tft.println("1. Header 1");
			tft.setCursor(5, 45);
			tft.println("2. Header 2");
			tft.setCursor(5, 60);
			tft.println("3. Gap Pulse");
			tft.setCursor(5, 75);
			tft.println("4. 0 Pulse");
			tft.setCursor(5, 90);
			tft.println("5. 1 Pulse");
			tft.setCursor(5, 105);
			tft.println("6. Pulse Space");

			int IRHeli_1_Pulse = 545;
			int IRHeli_0_Pulse = 270;
			int IRHeli_Gap_Pulse = 300;
			int IRHeli_Header_1 = 400;
			int IRHeli_Header_2 = 1640;
			int IRHeli_Pulse_Space = 136;
			int IRHeli_Menu = 0;
			unsigned int IR_Data_Packet[31];
				IR_Data_Packet[30]=0;
			IR_Data_Packet[31]=0;
			unsigned int RAW_IR[58];

			// Initialise the encoders
			Encoder1.write(127);
			Encoder2.write(28);

			while(IR_Heli_Active==2){

				Check_Joys();
				Check_Encoders();

				if ((digitalRead(_VQS_V1))==LOW){IRHeli_Menu = 0;while ((digitalRead(_VQS_V1)) == LOW){}}
				if ((digitalRead(_VQS_V2))==LOW){IRHeli_Menu = 1;while ((digitalRead(_VQS_V2)) == LOW){}}
				if ((digitalRead(_VQS_V3))==LOW){IRHeli_Menu = 2;while ((digitalRead(_VQS_V3)) == LOW){}}
				if ((digitalRead(_VQS_V4))==LOW){IRHeli_Menu = 3;while ((digitalRead(_VQS_V4)) == LOW){}}
				if ((digitalRead(_VQS_V5))==LOW){IRHeli_Menu = 4;while ((digitalRead(_VQS_V5)) == LOW){}}
				if ((digitalRead(_VQS_V6))==LOW){IRHeli_Menu = 5;while ((digitalRead(_VQS_V6)) == LOW){}}

				switch (IRHeli_Menu) {
				case 0:{
					tft.setTextColor(BLACK);
					tft.setCursor(180, 30);
					tft.println(IRHeli_Header_1);
  					if ((digitalRead(_S_Up))==LOW)
    				{
						IRHeli_Header_1 = IRHeli_Header_1+5;
     	 				while ((digitalRead(_S_Up)) == LOW){}
    				}
    				else if ((digitalRead(_S_Down))==LOW)
					{
						IRHeli_Header_1 = IRHeli_Header_1-5;
			     		while ((digitalRead(_S_Down)) == LOW){}
    				}
    				tft.setTextColor(GREEN);
					tft.setCursor(180, 30);
					tft.println(IRHeli_Header_1);
					break;
					}
					case 1:{
						tft.setTextColor(BLACK);
						tft.setCursor(180, 45);
						tft.println(IRHeli_Header_2);
  						if ((digitalRead(_S_Up))==LOW)
    					{
							IRHeli_Header_2 = IRHeli_Header_2+5;
     	 					while ((digitalRead(_S_Up)) == LOW){}
    					}
    					else if ((digitalRead(_S_Down))==LOW)
						{
							IRHeli_Header_2 = IRHeli_Header_2-5;
			     			while ((digitalRead(_S_Down)) == LOW){}
    					}
    					tft.setTextColor(GREEN);
						tft.setCursor(180, 45);
						tft.println(IRHeli_Header_2);
						break;
					}
					case 2:{
						tft.setTextColor(BLACK);
						tft.setCursor(180, 60);
						tft.println(IRHeli_Gap_Pulse);
  						if ((digitalRead(_S_Up))==LOW)
    					{
							IRHeli_Gap_Pulse = IRHeli_Gap_Pulse+5;
     	 					while ((digitalRead(_S_Up)) == LOW){}
    					}
    					else if ((digitalRead(_S_Down))==LOW)
						{
							IRHeli_Gap_Pulse = IRHeli_Gap_Pulse-5;
			     			while ((digitalRead(_S_Down)) == LOW){}
    					}
    					tft.setTextColor(GREEN);
						tft.setCursor(180, 60);
						tft.println(IRHeli_Gap_Pulse);
						break;
					}
					case 3:{
						tft.setTextColor(BLACK);
						tft.setCursor(180, 75);
						tft.println(IRHeli_0_Pulse);
  						if ((digitalRead(_S_Up))==LOW)
    					{
							IRHeli_0_Pulse = IRHeli_0_Pulse+5;
     	 					while ((digitalRead(_S_Up)) == LOW){}
    					}
    					else if ((digitalRead(_S_Down))==LOW)
						{
							IRHeli_0_Pulse = IRHeli_0_Pulse-5;
			    	 		while ((digitalRead(_S_Down)) == LOW){}
    					}
    					tft.setTextColor(GREEN);
						tft.setCursor(180, 75);
						tft.println(IRHeli_0_Pulse);
						break;
					}
					case 4:{
						tft.setTextColor(BLACK);
						tft.setCursor(180, 90);
						tft.println(IRHeli_1_Pulse);
  						if ((digitalRead(_S_Up))==LOW)
    					{
							IRHeli_1_Pulse = IRHeli_1_Pulse+5;
     	 					while ((digitalRead(_S_Up)) == LOW){}
    					}
    					else if ((digitalRead(_S_Down))==LOW)
						{
							IRHeli_1_Pulse = IRHeli_1_Pulse-5;
			     			while ((digitalRead(_S_Down)) == LOW){}
    					}
    					tft.setTextColor(GREEN);
						tft.setCursor(180, 90);
						tft.println(IRHeli_1_Pulse);
						break;
					}
					case 5:{
						tft.setTextColor(BLACK);
						tft.setCursor(180, 105);
						tft.println(IRHeli_Pulse_Space);
  						if ((digitalRead(_S_Up))==LOW)
    					{
							IRHeli_Pulse_Space = IRHeli_Pulse_Space+5;
     	 					while ((digitalRead(_S_Up)) == LOW){}
    					}
    					else if ((digitalRead(_S_Down))==LOW)
						{
							IRHeli_Pulse_Space = IRHeli_Pulse_Space-5;
			    	 		while ((digitalRead(_S_Down)) == LOW){}
    					}
    					tft.setTextColor(GREEN);
						tft.setCursor(180, 105);
						tft.println(IRHeli_Pulse_Space);
						break;
					}
				}

				IRHeli_Throttle = map(Encoder1NewPosition, 0, 255, 0, 127);

				IRHeli_Pitch = map(command[3], 0, 255, 0, 63);


				if(command[4]>140){
					IRHeli_Yaw = map(command[4], 140, 255, 9, 15);
				}
				else if(command[4]<114){
					IRHeli_Yaw = map(command[4], 0, 114, 0, 7);
				}
				else{
					IRHeli_Yaw = 8;
				}

				if(Encoder2NewPosition>29){
					IRHeli_Trim = map(Encoder2NewPosition, 29, 51, 29, 63);
				}
				else if(Encoder2NewPosition<22){
					IRHeli_Trim = map(Encoder2NewPosition, 0, 22, 0, 27);
				}
				else{
					IRHeli_Trim = 28;
				}

				// Channel
				IR_Data_Packet[0]=1;
				IR_Data_Packet[1]=0;
				IR_Data_Packet[2]=0;
				IR_Data_Packet[3]=0;

				// Pitch Data
				if(bitRead(IRHeli_Pitch,3)==0){IR_Data_Packet[7]=0;}else{IR_Data_Packet[7]=1;}
				if(bitRead(IRHeli_Pitch,2)==0){IR_Data_Packet[6]=0;}else{IR_Data_Packet[6]=1;}
				if(bitRead(IRHeli_Pitch,1)==0){IR_Data_Packet[5]=0;}else{IR_Data_Packet[5]=1;}
				if(bitRead(IRHeli_Pitch,0)==0){IR_Data_Packet[4]=0;}else{IR_Data_Packet[4]=1;}

				// Throttle Data
				if(bitRead(IRHeli_Throttle,7)==0){IR_Data_Packet[15]=0;}else{IR_Data_Packet[15]=1;}
				if(bitRead(IRHeli_Throttle,6)==0){IR_Data_Packet[14]=0;}else{IR_Data_Packet[14]=1;}
				if(bitRead(IRHeli_Throttle,5)==0){IR_Data_Packet[13]=0;}else{IR_Data_Packet[13]=1;}
				if(bitRead(IRHeli_Throttle,4)==0){IR_Data_Packet[12]=0;}else{IR_Data_Packet[12]=1;}
				if(bitRead(IRHeli_Throttle,3)==0){IR_Data_Packet[11]=0;}else{IR_Data_Packet[11]=1;}
				if(bitRead(IRHeli_Throttle,2)==0){IR_Data_Packet[10]=0;}else{IR_Data_Packet[10]=1;}
				if(bitRead(IRHeli_Throttle,1)==0){IR_Data_Packet[9]=0;}else{IR_Data_Packet[9]=1;}
				if(bitRead(IRHeli_Throttle,0)==0){IR_Data_Packet[8]=0;}else{IR_Data_Packet[8]=1;}

				// Yaw Data
				if(bitRead(IRHeli_Yaw,3)==0){IR_Data_Packet[19]=0;}else{IR_Data_Packet[19]=1;}
				if(bitRead(IRHeli_Yaw,2)==0){IR_Data_Packet[18]=0;}else{IR_Data_Packet[18]=1;}
				if(bitRead(IRHeli_Yaw,1)==0){IR_Data_Packet[17]=0;}else{IR_Data_Packet[17]=1;}
				if(bitRead(IRHeli_Yaw,0)==0){IR_Data_Packet[16]=0;}else{IR_Data_Packet[16]=1;}

				// Trim Data
				if(bitRead(IRHeli_Trim,5)==0){IR_Data_Packet[29]=0;}else{IR_Data_Packet[29]=1;}
				if(bitRead(IRHeli_Trim,4)==0){IR_Data_Packet[28]=0;}else{IR_Data_Packet[28]=1;}
				if(bitRead(IRHeli_Trim,3)==0){IR_Data_Packet[27]=0;}else{IR_Data_Packet[27]=1;}
				if(bitRead(IRHeli_Trim,2)==0){IR_Data_Packet[26]=0;}else{IR_Data_Packet[26]=1;}
				if(bitRead(IRHeli_Trim,1)==0){IR_Data_Packet[25]=0;}else{IR_Data_Packet[25]=1;}
				if(bitRead(IRHeli_Trim,0)==0){IR_Data_Packet[24]=0;}else{IR_Data_Packet[24]=1;}

				int i;

 				int checksum[10];
 				checksum[0] = 0;
 				checksum[1] = 0;
 				checksum[2] = 0;
 				checksum[3] = 0;

 				// compute checksum -- bitwise XOR of 4-bit chuncks
 				// with two zeros padding the *end* of the last two bits
 				for (i=0; i<32; i+=4)
 				{
  	 				// XOR
   					checksum[0] = IR_Data_Packet[i + 0] ^ checksum[0]; // the "^" operator is bitwise XOR (exclusive OR)
   					checksum[1] = IR_Data_Packet[i + 1] ^ checksum[1];
   					checksum[2] = IR_Data_Packet[i + 2] ^ checksum[2];
   					checksum[3] = IR_Data_Packet[i + 3] ^ checksum[3];
 				}

 				// now set bits 21-24 (array values 20-23) to the checksum
				IR_Data_Packet[20] = checksum[0];
				IR_Data_Packet[21] = checksum[1];
				IR_Data_Packet[22] = checksum[2];
				IR_Data_Packet[23] = checksum[3];


				for (i=0; i<58; i++){
					if ( (i & 0x01) == 0) {
						if(IR_Data_Packet[(i/2)]==1){
							RAW_IR[i]=IRHeli_1_Pulse;
						}
						else{
							RAW_IR[i]=IRHeli_0_Pulse;
						}
					}
					else{
						RAW_IR[i]=IRHeli_Gap_Pulse;
					}
				}

				irsend.sendRaw(RAW_IR,58,62); // Sony TV power code

				delay(IRHeli_Pulse_Space);

				if ((digitalRead(_S_Back))==LOW)
				{
					IR_Heli_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();
					Encoder1_Limit=51;

					// Wait for switch to be released
					while ((digitalRead(_S_Back)) == LOW){}
    			}
			}
			break;
		}
	}
}


/*******************************************************************/
/*         Functions used to Control Robots and Drones             */
/*******************************************************************/

void RCTractorControllerV2::Start_Humanoid(){
	while(Humanoid_Active!=0){

		if ((digitalRead(_S_Up))==LOW)
				{
					Serial.write(0x01);

					// Wait for switch to be released
					while ((digitalRead(_S_Up)) == LOW){}
		}

				if ((digitalRead(_S_Down))==LOW)
						{
							Serial.write(0x02);

							// Wait for switch to be released
							while ((digitalRead(_S_Down)) == LOW){}
		}

		if ((digitalRead(_S_Back))==LOW)
		{
			Humanoid_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

			// Wait for switch to be released
			while ((digitalRead(_S_Back)) == LOW){}
		}
	}
}


/*******************************************************************/
/*               Functions used to Control a TV                    */
/*******************************************************************/

void RCTractorControllerV2::Start_TV(){
	tft.fillRect(0,0,240,320,BLACK);
	tft.setTextColor(GREEN);
  			tft.setTextSize(2);
    		tft.setCursor(5, 5);

 switch (TV_Active) {
		    case 1:{
				tft.println(TV1.getName());
 			break;
			}
		    case 2:{
				tft.println(TV2.getName());
 			break;
			}
		    case 3:{
				tft.println(TV3.getName());
 			break;
			}
		    case 4:{
				tft.println(TV4.getName());
 			break;
			}
		    case 5:{
				tft.println(TV5.getName());
 			break;
			}
		    case 6:{
				tft.println(TV6.getName());
 			break;
			}
		    case 7:{
				tft.println(TV7.getName());
 			break;
			}
		    case 8:{
				tft.println(TV8.getName());
 			break;
			}
		    case 9:{
				tft.println(TV9.getName());
 			break;
			}
		    case 10:{
				tft.println(TV10.getName());
 			break;
			}

}

			    tft.fillRect(50,150,5,30,WHITE);
			    tft.fillRect(70,150,5,30,WHITE);
    tft.fillTriangle(170+30,165,170,165-15,170,165+15,GREEN);

	while(TV_Active!=0){
		Check_Joys();
		Check_Encoders();

		switch (TV_Active) {
		    case 1:{
				if(command[2]>200){
		      		irsend.sendNEC(0xFF10EF, 32); //
				}
				else if(command[2]<55){
					irsend.sendNEC(0xFF40BF, 32); //
				}
				else if(command[4]>200){
					irsend.sendNEC(0xFF609F, 32); //
				}
				else if(command[4]<55){
					irsend.sendNEC(0xFF20DF, 32); //
				}
			break;
			}

		    case 2:{
				if(command[2]>200){
		      		irsend.sendNEC(0xBD08F7, 32); //
				}
				else if(command[2]<55){
					irsend.sendNEC(0xBD30CF, 32); //
				}
				else if(command[4]>200){
					irsend.sendNEC(0xBD18E7, 32); //
				}
				else if(command[4]<55){
					irsend.sendNEC(0xBD38C7, 32); //
				}
			break;
			}

		    case 3:{
				if(command[2]>200){
		      		irsend.sendNEC(0xBD08F7, 32); //
				}
				else if(command[2]<55){
					irsend.sendNEC(0xBD30CF, 32); //
				}
				else if(command[4]>200){
					irsend.sendNEC(0xC00020, 32); //
				}
				else if(command[4]<55){
					irsend.sendNEC(0xC00021, 32); //
				}
			break;
			}

		    case 4:{
				if(command[2]>150){
		      		irsend.sendRC5(0x59, 12); //
				}
				else if(command[2]<100){
					irsend.sendRC5(0x859, 12); //
				}

				if(command[4]>150){
					irsend.sendRC5(0x50, 12); //
				}
				else if(command[4]<100){
					irsend.sendRC5(0x51, 12); //
				}
			break;
			}

		    default:{

		      TV_Active = 0;
		    break;
			}
		}

		if ((digitalRead(_S_Back))==LOW)
        {
			TV_Active = 0;
			menu.moveUp();
			Menu_Change = 1;
			menu.moveDown();

          	// Wait for switch to be released
          	while ((digitalRead(_S_Back)) == LOW){}
    	}
	}
}


/*******************************************************************/
/*                   Functions used by Games                       */
/*******************************************************************/

void RCTractorControllerV2::Start_Game(){
	tft.fillRect(0,0,240,320,BLACK);
	switch (Game_Active) {
	    case 1:{
			/*
  			Esplora TFT Snake

  			this is an open source version of the classic snake game.
  			Press one of the switches to change the direction of the
  			snake and move the slider to make the game faster or slower.
  			The score is in the upper right corner of the screen.
  			You earn 9 point for each beacon the snake eats. Enjoy :)

  			Created on 16 september 2013
  			By Corrado Possieri

  			Modified 13 January 2016 by Oisin O'Conchubhair
			*/

			snakeLength = snakeInitialLength;
			score = (snakeLength - snakeInitialLength) * 9;

			// Reset Encoder to 0
  			Encoder1.write(0);

  			// draw these positions
  			for(int ii = 0; ii < snakeLength; ii++) {
    			positions[0][ii] = startX + ii;
    			positions[1][ii] = startY;
    			// draw some boxes to figure where is the snake
    			tft.fillRect(scale*positions[0][ii], scale*positions[1][ii], scale, scale,WHITE); // the snake dimension are proportional to scale
  			}
  			delay(2000);

  			// to make the position of the beacon random
  			// use random seed with unconnected analog pin
  			randomSeed(analogRead(A11));

  			// place the beacon
  			placeBeacon();

  			// write the score
  			// convert score into String
  			String scoreString = String(score);
  			// convert string into char array
  			char charScore [4];
  			scoreString.toCharArray(charScore, 4);
  			// print it
  			tft.setTextColor(GREEN);
  			tft.setTextSize(2);
    		tft.setCursor(scorePosition, 0);
    		tft.println(charScore);

			while(Game_Active==1){

				// the snake moves accordingly to the value read from the encoder
  				Encoder1NewPosition = Encoder1.read();

 				if (Encoder1NewPosition != Encoder1OldPosition) {

					if(Encoder1NewPosition<0){
  						Encoder1NewPosition=0;
  						Encoder1.write(Encoder1NewPosition);
					}
					if(Encoder1NewPosition>51){
  						Encoder1NewPosition=51;
   	 					Encoder1.write(Encoder1NewPosition);
					}

    			Encoder1OldPosition = Encoder1NewPosition;

 				slider = (Encoder1NewPosition*5);
				}

  				// map this value in a value of time
  				unsigned int timeDelay = map(slider, 0, 1023, 50, 1000);
  				if(millis() - lastTimeMoved > timeDelay) {
    				// let the snake move
    				moveTheSnake();
    				lastTimeMoved = millis();
  				}

    			paddleY = map(analogRead(_JX_L), 0, 1023, 0, 255);
  				paddleX = map(analogRead(_JY_L), 0, 1023, 0, 255);

  				// read the buttons to change directions
  				if (paddleX < 55 && directionSnake != 3) {
    				// if the snake is going up it can't go down
    				// so we use that AND funciotn
    				directionSnake = 1;
  				}
  				if (paddleY < 55 && directionSnake != 0) {
    				// if the snake is going up it can't go down
    				// so we use that AND funciotn
    				directionSnake = 2;
  				}
  				if (paddleX > 200 && directionSnake != 1) {
  	 	 			// if the snake is going up it can't go down
    				// so we use that AND funciotn
    				directionSnake = 3;
  				}
  				if (paddleY > 200 && directionSnake != 2) {
    				// if the snake is going up it can't go down
    				// so we use that AND funciotn
    				directionSnake = 0;
  				}

  				// see it we hurt another part of the snake
  				for(int kk = 0; kk < snakeLength - 1 ; kk++) {
    				if(positions[0][kk] == positions[0][snakeLength - 1] &&
     	 				positions[1][kk] == positions[1][snakeLength - 1]) {
        				// draw a blue blackground
    					tft.fillRect(0, 0, 120, 20,BLUE);
        				// write in white
        	  			tft.setTextColor(WHITE);
        				// select the text size
  						tft.setTextSize(2);
        				// "you Lose"
        	    		tft.setCursor(0, 0);
    					tft.println("Game Over");

						delay(5000);

        				// Exit the game
        				Game_Active = 0;
						menu.moveUp();
						Menu_Change = 1;
						menu.moveDown();
      				}
  				}

  				// update the score
  				score = (snakeLength - snakeInitialLength) * 9;
  				// if it is different from previous one
  				if(score != lastScore) {
  	  				// delete the old one
   					tft.setTextColor(BLACK);
    				// convert score into String
    				String scoreString = String(lastScore);
    				// convert string into char array
    				char charScore [4];
    				scoreString.toCharArray(charScore, 4);
    				// print it
     				tft.setCursor(scorePosition, 0);
    				tft.println(charScore);
    				// print the new one
   					tft.setTextColor(GREEN);
    				// convert score into String
    				scoreString = String(score);
    				// convert string into char array
    				scoreString.toCharArray(charScore, 4);
    				// print it
     				tft.setCursor(scorePosition, 0);
    				tft.println(charScore);
    				// save the value of last score
    				lastScore = score;
  				}

  				// a little pause for stability
  				delay(10);


				if ((digitalRead(_S_Back))==LOW)
        		{
					Game_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

          			// Wait for switch to be released
          			while ((digitalRead(_S_Back)) == LOW){}
    				}
				}

	    break;
}

	    case 2:{
		    /*
			 TFT Pong
			 This example for the Arduino screen reads the values
			 of 2 potentiometers to move a rectangular platform
			 on the x and y axes. The platform can intersect
			 with a ball causing it to bounce.

			 This example code is in the public domain.

			 Created by Tom Igoe December 2012
			 Modified 15 April 2013 by Scott Fitzgerald
			 Modified 13 January 2016 by Oisin O'Conchubhair

			 http://www.arduino.cc/en/Tutorial/TFTPong
	 		*/

			while(Game_Active==2){

				// save the width and height of the screen
			  	int myWidth = tft.width()-100;
  				int myHeight = tft.height()-160;

  				// map the paddle's location to the position of the potentiometers
  				paddleX = map(analogRead(_JX_L), 0, 1023, 0, myWidth) - 20 / 2;
  				paddleY = map(analogRead(_JY_L), 0, 1023, 0, myHeight) - 5 / 2;

  				// set the fill color to black and erase the previous
  				// position of the paddle if different from present

  				if (oldPaddleX != paddleX || oldPaddleY != paddleY) {
    				tft.fillRect(oldPaddleX, oldPaddleY, 20, 5, BLACK);
  				}

  				// draw the paddle on screen, save the current position
  				// as the previous.

  				tft.fillRect(paddleX, paddleY, 20, 5, WHITE);
  				oldPaddleX = paddleX;
  				oldPaddleY = paddleY;

  				// update the ball's position and draw it on screen
  				if (millis() % ballSpeed < 2) {
    				moveBall();
  				}

				if ((digitalRead(_S_Back))==LOW)
        		{
					Game_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

          			// Wait for switch to be released
          			while ((digitalRead(_S_Back)) == LOW){}
    			}
			}

	    break;
}

		case 3:{
			/*
			TFT Brick with two pot1entiometers
			Reads the value of one 10k pot1entiometer to move a paddle that
			will bounce a ball toward a row of bricks. If the bricks are
			hit by the ball they will break. Once all bricks are destroyed
			the game is over and you win.

			Created by christopherJD 7 October 2015
			 Modified 13 January 2016 by Oisin O'Conchubhair
			*/
			paddleY = 123;
			ballDirectionX = 1;
			ballDirectionY = 1;
			ballSpeed = 10; // Lower is faster
			ballX = 0; //middle of screen
			ballY = 40; //below bricks
			oldBallX = 0;
			oldBallY = 0;

  intro();
  Brickmenu();
  initGame();

			while(Game_Active==3){

  			//Lose Screen
  			if (numberLives <= 0){
  	  			loseScreen();
  			}

  			//sync the paddle and pot1entiometers location
  			paddle = map(analogRead(_JX_L), 1024, -128, 0, screenWidth);
  			paddle2 = paddle + paddleSection;
  			paddle3 = paddle2 + paddleSection;
  			paddle4 = paddle3 + paddleSection;
  			paddle5 = paddle4 + paddleSection;

  			//draws the paddle using a rectangle (this is when it is moving)
  			if (oldPaddle != paddle){
  	 			tft.fillRect(oldPaddle, paddleY, paddleWidth, paddleHeight, BLACK);
  			}

  			//draws paddle (if not moving)
  			tft.fillRect(paddle, paddleY, paddleWidth, paddleHeight, GREEN);
  			oldPaddle = paddle;

  			//updated the balls position
  			if (millis() % ballSpeed < 2){
    			BrickmoveBall();
  			}

			if ((digitalRead(_S_Back))==LOW)
        		{
					Game_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

          			// Wait for switch to be released
          			while ((digitalRead(_S_Back)) == LOW){}
    			}
			}

	    break;
		}

	    default:{

	      Game_Active = 0;
	    break;
	}
  }
}


// Functions used by Snake Game

void RCTractorControllerV2::moveTheSnake() {
  // move the snake according to the direction
  switch(directionSnake) {
    case 0: { // move right
      if(beaconIsEaten()) { // if we ate the beacon
        snakeLength++; // the snake increases in length
        positions[0][snakeLength - 1] =
          (positions[0][snakeLength - 2] + 1) % xScreen;
        // we use the modulus so when the snake goes out from a part
        // it returns in the opposite one
        // the y is the same as before
        positions[1][snakeLength - 1] = positions[1][snakeLength - 2];
        // draw the new head of the snake
        drawSnakeHead();
      } else {
        // update positions
        updatePositions();
        // move also the terminal of the snake
        positions[0][snakeLength - 1] =
          (positions[0][snakeLength - 1] + 1) % xScreen;
        // draw the new head
        drawSnakeHead();
      }
      break;
    }

    case 1: { // move down
      if(beaconIsEaten()) { // if we ate the beacon
        snakeLength++; // the snake increases in length
        positions[1][snakeLength - 1] =
          (positions[1][snakeLength - 2] + 1) % yScreen;
        positions[0][snakeLength - 1] = positions[0][snakeLength - 2];
        // draw the new head of the snake
        drawSnakeHead();
      } else {
        // update positions
        updatePositions();
        // move also the terminal of the snake
        positions[1][snakeLength - 1] =
          (positions[1][snakeLength - 1] + 1) % yScreen;
        // draw the new head
        drawSnakeHead();
      }
      break;
    }

    case 2: { // move left
      if(beaconIsEaten()) { // if we ate the beacon
        snakeLength++; // the snake increases in length
        positions[0][snakeLength - 1] =
          (positions[0][snakeLength - 2] - 1);
        // if it goes utside replace on the other part
        if(positions[0][snakeLength - 1] <= 0) {
          positions[0][snakeLength - 1] = xScreen - 1;
        }
        positions[1][snakeLength - 1] = positions[1][snakeLength - 2];
        // draw the new head of the snake
        drawSnakeHead();
      } else {
        // update positions
        updatePositions();
        // move also the terminal of the snake
        positions[0][snakeLength - 1] =
          (positions[0][snakeLength - 1] - 1);
        if(positions[0][snakeLength - 1] <= 0) {
          positions[0][snakeLength - 1] = xScreen - 1;
        }
        // draw the new head
        drawSnakeHead();
      }
      break;
    }

    case 3: { // move up
      if(beaconIsEaten()) { // if we ate the beacon
        snakeLength++; // the snake increases in length
        positions[1][snakeLength - 1] =
          (positions[1][snakeLength - 2] - 1);
        if(positions[1][snakeLength - 1] <= 0) {
          positions[1][snakeLength - 1] = yScreen - 1;
        }
        positions[0][snakeLength - 1] = positions[0][snakeLength - 2];
        // draw the new head of the snake
        drawSnakeHead();
      } else {
        // update positions
        updatePositions();
        // move also the terminal of the snake
        positions[1][snakeLength - 1] =
          (positions[1][snakeLength - 1] - 1);
        if(positions[1][snakeLength - 1] <= 0) {
          positions[1][snakeLength - 1] = yScreen - 1;
        }
        // draw the new head
        drawSnakeHead();
      }
      break;
    }
  }
}

boolean RCTractorControllerV2::beaconIsEaten() {
  // this function returns true if beacon is eated and
  // false if it is not
  if(positions[0][snakeLength - 1] == beacon[0] &&
    positions[1][snakeLength - 1] == beacon[1]) {
    // put the beacon in another place
    placeBeacon();
    return true;
  } else {
    return false;
  }
}

void RCTractorControllerV2::updatePositions() {
  // this function update the position of the snake except the
  // head of the snake, which position is user choice dependent
  // make the snake move
  // first we have to delete the tail of the snake
  tft.fillRect(scale * positions[0][0], scale * positions[1][0], scale, scale, BLACK);
  // next update positions
  for(int ii = 0; ii < snakeLength - 1; ii++) {
    // update the positions
    positions[0][ii] = positions[0][ii + 1];
    positions[1][ii] = positions[1][ii + 1];
  }
}

void RCTractorControllerV2::drawSnakeHead() {
  // this function draw the new head of the snake
 tft.fillRect(scale*positions[0][snakeLength - 1], scale*positions[1][snakeLength - 1], scale, scale, WHITE);
}

void RCTractorControllerV2::placeBeacon() {
  // this function place the beacon where
  // there is not a snake part
  // repeat while we don't break it
  while (true) {
    // define an int for verification
    int flag = 0;
    // don't put the beacon on the external of the screen
    const int tollerance = 5;
    // locate the beacon in a random place
    beacon[0] = (int) random(tollerance, xScreen - tollerance);
    beacon[1] = (int) random(tollerance, yScreen - tollerance);
    // find if a part of the snake is on the beacon
    for(int jj = 0; jj < snakeLength; jj++) {
      // if they have the same x
      if(beacon[0] == positions[0][jj]) {
        // cheack the y
        if(beacon[1] == positions[1][jj]) {
          // increment the flag
          flag++;
        }
      }
    }
    // if the flag is 0 we don't have the beacon over
    // a part of the snake
    if(flag == 0) {
      // so we can go away
      break;
    }
  }

  // draw the beacon
  tft.fillRect(scale * beacon[0], scale * beacon[1], scale, scale, RED);
}





// Functions used by Pong Game

// this function determines the ball's position on screen
void RCTractorControllerV2::moveBall() {
  // if the ball goes offscreen, reverse the direction:
  if (ballX > (tft.width()-100) || ballX < 0) {
    ballDirectionX = -ballDirectionX;
  }

  if (ballY > (tft.height()-160) || ballY < 0) {
    ballDirectionY = -ballDirectionY;
  }

  // check if the ball and the paddle occupy the same space on screen
  if (inPaddle(ballX, ballY, paddleX, paddleY, 20, 5)) {
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
  }

  // update the ball's position
  ballX += ballDirectionX;
  ballY += ballDirectionY;

  // erase the ball's previous position

  if (oldBallX != ballX || oldBallY != ballY) {
    tft.fillRect(oldBallX, oldBallY, 5, 5, BLACK);
  }


  // draw the ball's current position
  tft.fillRect(ballX, ballY, 5, 5,WHITE);

  oldBallX = ballX;
  oldBallY = ballY;

}

// this function checks the position of the ball
// to see if it intersects with the paddle
boolean RCTractorControllerV2::inPaddle(int x, int y, int rectX, int rectY, int rectWidth, int rectHeight) {
  boolean result = false;

  if ((x >= rectX && x <= (rectX + rectWidth)) &&
      (y >= rectY && y <= (rectY + rectHeight))) {
    result = true;
  }

  return result;
}

// Functions used by Brick Game

/*
A warm Welcome before the harsh and relentless game of brick
Players be warned before the inevidable struggle
*/
void RCTractorControllerV2::intro()
{
  tft.setTextSize(4); //guess
  tft.setCursor(0,0);
  tft.println("BRICK BREAK");
  tft.setTextSize(1);
  tft.println("by: pepperpwnipizza");
  tft.println(" ");
  tft.println("Break the bricks");
  tft.println("You have 3 Lives");
  delay(5000);
	tft.fillRect(0,0,240,320,BLACK);
}

void RCTractorControllerV2::Brickmenu(){

  while (start == false){

    // Text on The screen
    tft.setTextSize(2);
    tft.setCursor(0,0);
    tft.println("BREAK");
    tft.println("DEM");
    tft.println("BRICKS");
    tft.setTextSize(1);
    tft.setCursor(80,60);
    tft.println("Play Game");
    tft.setCursor(80,70);
    tft.println("Settings");
    tft.setCursor(80,80);
    tft.println("Stats");
    ////////////////////////////

    //Cursor on screen
    mouseX = map(analogRead(_JX_L), 1023, -128, 0, screenWidth);
    mouseY = map(analogRead(_JY_L), 1023, -128, 0, screenHeight);

    if (oldMouseX != mouseX || oldMouseY != mouseY){
    triangleErase(oldMouseX, oldMouseY);
    }
    triangle(mouseX, mouseY);
    oldMouseX = mouseX;
    oldMouseY = mouseY;
    //////////////////////////////////////////////////////////

    //Changes cursor to red when inside text and allows you to select play game
    if (cursorInText(mouseX, mouseY, 80, 60, 60, 5)){
      while(mouseSelect == 0){
        if (cursorInText(mouseX, mouseY, 80, 60, 60, 5) == false){
          Brickmenu();
        }
        // Text on The screen
        tft.setTextSize(2);
        tft.setCursor(0,0);
        tft.println("BREAK");
        tft.println("DEM");
        tft.println("BRICKS");
        tft.setTextSize(1);
        tft.setCursor(80,60);
        tft.println("Play Game");
        tft.setCursor(80,70);
        tft.println("Settings");
        tft.setCursor(80,80);
        tft.println("Stats");
        ////////////////////////////
        //Cursor on screen
        mouseX = map(analogRead(_JX_L), 1023, -128, 0, screenWidth);
        mouseY = map(analogRead(_JY_L), 1023, -128, 0, screenHeight);
        mouseSelect = map(analogRead(_JY_R), 0, 1023, 0, 1); //allows you to select things

        if (oldMouseX != mouseX || oldMouseY != mouseY){
        triangleErase(oldMouseX, oldMouseY);
        }
        triangleRed(mouseX, mouseY);
        oldMouseX = mouseX;
        oldMouseY = mouseY;
        //////////////////////////////////////////////////////////
      }

	tft.fillRect(0,0,240,320,BLACK);
      tft.setCursor(40,40);
      tft.setTextSize(4);
      tft.println("GLHF");
      delay(2000);
	tft.fillRect(0,0,240,320,BLACK);
      start = true;
    }

    //Checks Settings Selection
    if (cursorInText(mouseX, mouseY, 80, 70, 60, 5)){
      while(mouseSelect == 0){
        if (cursorInText(mouseX, mouseY, 80, 70, 60, 5) == false){
          Brickmenu();
        }
        // Text on The screen
        tft.setTextSize(2);
        tft.setCursor(0,0);
        tft.println("BREAK");
        tft.println("DEM");
        tft.println("BRICKS");
        tft.setTextSize(1);
        tft.setCursor(80,60);
        tft.println("Play Game");
        tft.setCursor(80,70);
        tft.println("Settings");
        tft.setCursor(80,80);
        tft.println("Stats");
        ////////////////////////////
        //Cursor on screen
        mouseX = map(analogRead(_JX_L), 1023, -128, 0, screenWidth);
        mouseY = map(analogRead(_JY_L), 1023, -128, 0, screenHeight);
        mouseSelect = map(analogRead(_JY_R), 0, 1023, 0, 1); //allows you to select things

        if (oldMouseX != mouseX || oldMouseY != mouseY){
        triangleErase(oldMouseX, oldMouseY);
        }
        triangleRed(mouseX, mouseY);
        oldMouseX = mouseX;
        oldMouseY = mouseY;
        //////////////////////////////////////////////////////////
      }

      settings();
      start = true; //breaks the loop
    }
  }
}

/*
This is where you can change different game settings
*/
void RCTractorControllerV2::settings(){
	tft.fillRect(0,0,240,320,BLACK);
  boolean settingsScreen = true;
    while(settingsScreen == true){
      if(mouseSelect == 0){
        settingsScreen = false;
      }
      tft.setCursor(0,0);
      tft.print("Ball speed: ");
      mouseSelect = map(analogRead(_JY_R), 0, 1023, 0, 1); //allows you to select things
      mouseX = map(analogRead(_JX_L), 0, 1023, 0, 10);
      ballSpeed = 10 - mouseX; //this way the higher the number the faster the speed
      tft.setCursor(100,0);
      tft.print(ballSpeed);
      tft.fillRect(100,0,10,10,BLACK);
    }
	tft.fillRect(0,0,240,320,BLACK);
    Brickmenu();
}

/*
The bricks get drawn here and other inital game settings are set
*/
void RCTractorControllerV2::initGame()
{
  //reinitialize the brickHit array
  for(int i = 0; i < 18; i++){
    brickHit[i] = false;
  }
	tft.fillRect(0,0,240,320,BLACK);
  //reinitialize variables
  ////////////////////////////
  bricksBroken = 0;
  numberLives = 3;
  start = false;
  mouseSelect = 0;
  ////////////////////////////
  Brickmenu();
  characterInfo();
  bricks(); //draw bricks
}

void RCTractorControllerV2::characterInfo(){
  tft.fillRect(0,0,tft.width(),10,BLACK); //clear text
  tft.setTextSize(1);
  tft.setCursor(60,0);
  tft.print("Lives: ");
  tft.print(numberLives); //for some reason i cant use + on println wtf
}

/*
All the logic about the ball.
*/
void RCTractorControllerV2::BrickmoveBall()
{
  //prevents the ball from going off the screen
  if (ballX < 0 || ballX > tft.width()){
    ballDirectionX = -ballDirectionX;
  }
  if (ballY < 0){
    ballDirectionY = -ballDirectionY;
  }
  if (ballY >= tft.height()){
    numberLives = numberLives - 1;
    characterInfo();
    ballX = 0; //reset the ball
    ballY = 40;
    ballDirectionX = 1;
    ballDirectionY = 1;
  }

  //prevents the ball from going in the paddle moves ball left
  if(ballInPaddle(ballX, ballY, paddle, paddleY, paddleSection, paddleHeight) == true){
    ballDirectionX = -1;
    ballDirectionY = -1;
  }
   //prevents the ball from going in the paddle moves ball up
  if(ballInPaddle(ballX, ballY, paddle2, paddleY, paddleSection, paddleHeight) == true){
    ballDirectionX = -2;
    ballDirectionY = -1;
  }
   //prevents the ball from going in the paddle moves ball right
  if(ballInPaddle(ballX, ballY, paddle3, paddleY, paddleSection, paddleHeight) == true){
    ballDirectionY = -1;
    ballDirectionX = 0;
  }
   //prevents the ball from going in the paddle moves ball up
  if(ballInPaddle(ballX, ballY, paddle4, paddleY, paddleSection, paddleHeight) == true){
    ballDirectionX = 2;
    ballDirectionY = -1;
  }
   //prevents the ball from going in the paddle moves ball right
  if(ballInPaddle(ballX, ballY, paddle5, paddleY, paddleSection, paddleHeight) == true){
    ballDirectionY = -1;
    ballDirectionX = 1;
  }

  brickLogic();

  //update ball position
  ballX += ballDirectionX;
  ballY += ballDirectionY;

  //erase the balls previous position

  if (oldBallX != ballX || oldBallY != ballY){
    tft.fillRect(oldBallX, oldBallY, ballWidth, ballHeight,RED);
  }

  //draw the balls current position
  tft.fillRect(ballX, ballY, ballWidth, ballHeight,GREEN);
}
/*
determines if the bricks have been hit
*/
void RCTractorControllerV2::brickLogic(){
  ///////////////////////////////////////////////////////////////////////////////////////////////
//Breaks dem bricks
if(ballInBrick(ballX, ballY, brickNumber[0], 10, brickWidth, brickHeight) && brickHit[0] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[0], 10);
    brickHit[0] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[1], 10, brickWidth, brickHeight) && brickHit[1] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[1], 10);
    brickHit[1] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[2], 10, brickWidth, brickHeight) && brickHit[2] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[2], 10);
    brickHit[2] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[3], 10, brickWidth, brickHeight) && brickHit[3] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[3], 10);
    brickHit[3] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[4], 10, brickWidth, brickHeight) && brickHit[4] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[4], 10);
    brickHit[4] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[5], 10, brickWidth, brickHeight) && brickHit[5] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[5], 10);
    brickHit[5] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[6], 17, brickWidth, brickHeight) && brickHit[6] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[6], 17);
    brickHit[6] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[7], 17, brickWidth, brickHeight) && brickHit[7] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[7], 17);
    brickHit[7] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[8], 17, brickWidth, brickHeight) && brickHit[8] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[8], 17);
    brickHit[8] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[9], 17, brickWidth, brickHeight) && brickHit[9] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[9], 17);
    brickHit[9] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[10], 17, brickWidth, brickHeight) && brickHit[10] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[10], 17);
    brickHit[10] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[11], 17, brickWidth, brickHeight) && brickHit[11] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[11], 17);
    brickHit[11] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[12], 24, brickWidth, brickHeight) && brickHit[12] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[12], 24);
    brickHit[12] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[13], 24, brickWidth, brickHeight) && brickHit[13] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[13], 24);
    brickHit[13] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[14], 24, brickWidth, brickHeight) && brickHit[14] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[14], 24);
    brickHit[14] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[15], 24, brickWidth, brickHeight) && brickHit[15] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[15], 24);
    brickHit[15] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[16], 24, brickWidth, brickHeight) && brickHit[16] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[16], 24);
    brickHit[16] = true;
  }
if(ballInBrick(ballX, ballY, brickNumber[17], 24, brickWidth, brickHeight) && brickHit[17] == false){
    ballDirectionX = -ballDirectionX;
    ballDirectionY = -ballDirectionY;
    destroyBrick(brickNumber[17], 24);
    brickHit[17] = true;
  }
///////////////////////////////////////////////////////////////////////////////////////////////
}

/*
Checks to see if the ball and paddle hit each other
*/
boolean RCTractorControllerV2::ballInPaddle(int x, int y, int rectX, int rectY, int rectWidth, int rectHeight) {
  boolean result = false;

  if ((x >= rectX && x <= (rectX + rectWidth)) &&
      (y >= rectY && y <= (rectY + rectHeight))) {
       result = true;
      }

return result;
}

/*
The logic of the bricks. Draws them, destroys them if necessary
*/
void RCTractorControllerV2::bricks(){

  int i = 0;
  int j = 0;
  int k = 0;
  int x1 = 0; //location of brick to draw
  int x2 = 0;
  int x3 = 0;
  //first row
  for(i; i <= bricksPerRow - 1; i++){

    //row 1
    tft.fillRect(x1, 10, brickWidth, brickHeight,RED);
    brickNumber[i] = x1;
    x1 = x1 + brickWidth + gap;

    //row 2
    for (j; j <= bricksPerRow - 1; j++){
    tft.fillRect(x2, 17, brickWidth, brickHeight, YELLOW);
    brickNumber[j+6] = x2;
    x2 = x2 + brickWidth + gap;

    //row 3
      for (k; k<= bricksPerRow - 1; k++){
      tft.fillRect(x3, 24, brickWidth, brickHeight, BLUE);
      brickNumber[k+12] = x3;
      x3 = x3 + brickWidth + gap;
      }
    }
  }
}

/*
Checks to see if the ball hits a brick
*/
boolean RCTractorControllerV2::ballInBrick(int x, int y, int rectX, int rectY, int rectWidth, int rectHeight) {
  boolean result = false;

  if ((x >= rectX && x <= (rectX + rectWidth)) &&
      (y >= rectY && y <= (rectY + rectHeight))) {
       result = true;
      }

return result;
}

/*
Checks to see if the ball hits a brick
*/
boolean RCTractorControllerV2::cursorInText(int x, int y, int rectX, int rectY, int rectWidth, int rectHeight) {
  boolean result = false;

  if ((x >= rectX && x <= (rectX + rectWidth)) &&
      (y >= rectY && y <= (rectY + rectHeight))) {
       result = true;
      }

return result;
}

/*
Draws the cursor, which is a triangle
*/

void RCTractorControllerV2::triangle(int x, int y){

  //variables
  int botXL = x + sideLength; //the left side of the triangle
  int botYL = y + sideLength; //the left side of the triangle
  int botXR = x - sideLength; //the right side of the triangle
  int botYR = y + sideLength; //the right side of the triangle
  tft.drawLine(x, y, botXL, botYL, RED);
  tft.drawLine(x, y, botXR, botYR, RED);
  tft.drawLine(botXL, botYL, botXR, botYR, RED);
}

/*
Draws the cursor, which is a triangle this one is red
*/

void RCTractorControllerV2::triangleRed(int x, int y){

  //variables
  int botXL = x + sideLength; //the left side of the triangle
  int botYL = y + sideLength; //the left side of the triangle
  int botXR = x - sideLength; //the right side of the triangle
  int botYR = y + sideLength; //the right side of the triangle
  tft.drawLine(x, y, botXL, botYL, BLUE);
  tft.drawLine(x, y, botXR, botYR, BLUE);
  tft.drawLine(botXL, botYL, botXR, botYR, BLUE);
}

/*
Erases the cursor, which is a triangle
only changes the color, what a freakin waste
*/

void RCTractorControllerV2::triangleErase(int x, int y){

  //variables
  int botXL = x + sideLength; //the left side of the triangle
  int botYL = y + sideLength; //the left side of the triangle
  int botXR = x - sideLength; //the right side of the triangle
  int botYR = y + sideLength; //the right side of the triangle
  tft.drawLine(x, y, botXL, botYL, BLACK);
  tft.drawLine(x, y, botXR, botYR, BLACK);
  tft.drawLine(botXL, botYL, botXR, botYR, BLACK);
}

/*
Destroy a brick
*/
void RCTractorControllerV2::destroyBrick(int x, int y){
  tft.fillRect(x, y, brickWidth, brickHeight,BLACK);
}

/*
lose screen
*/
void RCTractorControllerV2::loseScreen(){
	tft.fillRect(0,0,240,320,BLACK);
  tft.setCursor(20,40);
  tft.setTextSize(4);
  tft.println("LOSER");
  delay(1000);
	tft.fillRect(0,0,240,320,BLACK);
  tft.setCursor(0,0);
  tft.setTextSize(4);
  tft.println("LOSER");
  delay(1000);
	tft.fillRect(0,0,240,320,BLACK);
  tft.setCursor(0,100);
  tft.setTextSize(4);
  tft.println("LOSER");
  delay(1000);
  initGame();
}

/*
win screen
*/
void RCTractorControllerV2::winScreen(){
	tft.fillRect(0,0,240,320,BLACK);
  tft.setCursor(20,40);
  tft.setTextSize(4);
  tft.println("WINNER");
  delay(500);
  initGame();
}

	/*******************************************************************/
	/*                   Functions used by Tools                       */
	/*******************************************************************/

void RCTractorControllerV2::Select_Tool(){

	tft.fillRect(0,0,240,320,BLACK);
		switch (Tool_Active) {
	    case 1:{
			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("joystick and encoder");
			tft.setCursor(5, 45);
			tft.println(" values in real time.");

			delay(1000);

			tft.setTextColor(BLACK);

			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("joystick and encoder");
			tft.setCursor(5, 45);
			tft.println(" values in real time.");

			tft.setTextColor(GREEN);

			tft.setCursor(5, 5);
			tft.println("Left Joystick X");
			tft.setCursor(5, 25);
			tft.println("Left Joystick Y");
			tft.setCursor(5, 45);
			tft.println("Right Joystick X");
			tft.setCursor(5, 65);
			tft.println("Right Joystick Y");
			tft.setCursor(5, 85);
			tft.println("Left Encoder 1");
			tft.setCursor(5, 105);
			tft.println("Left Encoder 2");
			tft.setCursor(5, 125);
			tft.println("Right Encoder 1");
			tft.setCursor(5, 145);
			tft.println("Right Encoder 2");


			while(Tool_Active==1){

				Check_Joys();
				Check_Encoders();

				int Number_pos = 200;

				if(old_command[1]!=command[1]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 5);
			    	tft.println(old_command[1]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 5);
			    	tft.println(command[1]);
				}

				if(old_command[2]!=command[2]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 25);
			    	tft.println(old_command[2]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 25);
			    	tft.println(command[2]);
				}

				if(old_command[3]!=command[3]){
  					tft.setTextColor(BLACK);
			   	 	tft.setCursor(Number_pos, 45);
			    	tft.println(old_command[3]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 45);
			    	tft.println(command[3]);
				}

				if(old_command[4]!=command[4]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 65);
			    	tft.println(old_command[4]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 65);
			    	tft.println(command[4]);
				}

				if(old_command[5]!=command[5]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 85);
			    	tft.println(old_command[5]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 85);
			    	tft.println(command[5]);
				}

				if(old_command[6]!=command[6]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 105);
			    	tft.println(old_command[6]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 105);
			    	tft.println(command[6]);
				}

				if(old_command[7]!=command[7]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 125);
			    	tft.println(old_command[7]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 125);
			    	tft.println(command[7]);
				}

				if(old_command[8]!=command[8]){
  					tft.setTextColor(BLACK);
			    	tft.setCursor(Number_pos, 145);
			    	tft.println(old_command[8]);
					tft.setTextColor(GREEN);
			    	tft.setCursor(Number_pos, 145);
			    	tft.println(command[8]);
				}

				for(int i=1;i<11;i++){
					old_command[i] = command[i];
			}

				if ((digitalRead(_S_Back))==LOW)
			    {
					Tool_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

			    	// Wait for switch to be released
			    	while ((digitalRead(_S_Back)) == LOW){}
			    }
			}
		break;
		}
				case 2:{
					int c_line = 122;
					tft.setTextColor(GREEN);
					tft.setTextSize(2);
					tft.setCursor(5, 5);
					tft.println("This tool displays");
					tft.setCursor(5, 25);
					tft.println("the current state");
					tft.setCursor(5, 45);
					tft.println("of each switch.");

					delay(1000);

					tft.setTextColor(BLACK);

					tft.setCursor(5, 5);
					tft.println("This tool displays");
					tft.setCursor(5, 25);
					tft.println("the current state");
					tft.setCursor(5, 45);
					tft.println("of each switch.");


					tft.setTextColor(GREEN);
					tft.setCursor(c_line-100, 290);
					tft.println("Push Back to Exit");

					while(Tool_Active==2){

						if ((digitalRead(_S_Left_Joy))==LOW)
					    {
							tft.setTextColor(GREEN);
							tft.setCursor(c_line-100, 20);
							tft.println("L_J");
						}
						else
					    {
							tft.setTextColor(BLUE);
							tft.setCursor(c_line-100, 20);
							tft.println("L_J");
						}

						if ((digitalRead(_S_EN1))==LOW)
					    {
							tft.setTextColor(GREEN);
							tft.setCursor(c_line-50, 20);
							tft.println("L_E");
						}
						else
					    {
							tft.setTextColor(BLUE);
							tft.setCursor(c_line-50, 20);
							tft.println("L_E");
						}

						if ((digitalRead(_S_Right_Joy))==LOW)
					    {
							tft.setTextColor(GREEN);
							tft.setCursor(c_line+12, 20);
							tft.println("R_J");
						}
						else
					    {
							tft.setTextColor(BLUE);
							tft.setCursor(c_line+12, 20);
							tft.println("R_J");
						}

						if ((digitalRead(_S_EN2))==LOW)
					    {
							tft.setTextColor(GREEN);
							tft.setCursor(c_line+62, 20);
							tft.println("R_E");
						}
						else
					    {
							tft.setTextColor(BLUE);
							tft.setCursor(c_line+62, 20);
							tft.println("R_E");
						}

							if ((digitalRead(_S_DB))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-100, 60);
								tft.println("Dip");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-100, 60);
								tft.println("Dip");
							}

							if ((digitalRead(_S_FB))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-25, 60);
								tft.println("Full");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-25, 60);
								tft.println("Full");
							}

							if ((digitalRead(_S_WL))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+63, 60);
								tft.println("Work");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+63, 60);
								tft.println("Work");
						}

								if ((digitalRead(_S_F1))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-90, 100);
								tft.println("Ex_1");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-90, 100);
								tft.println("Ex_1");
							}

							if ((digitalRead(_S_F2))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-25, 100);
								tft.println("Ex_2");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-25, 100);
								tft.println("Ex_2");
							}

							if ((digitalRead(_S_F3))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+40, 100);
								tft.println("Ex_3");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+40, 100);
								tft.println("Ex_3");
						}

						if ((digitalRead(_S_Up))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-98, 140);
								tft.println("Up");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-98, 140);
								tft.println("Up");
							}

							if ((digitalRead(_S_Down))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-45, 140);
								tft.println("Down");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-45, 140);
								tft.println("Down");
							}

							if ((digitalRead(_S_Select))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+30, 140);
								tft.println("Select");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+30, 140);
								tft.println("Select");
						}

							if ((digitalRead(_VQS_V1))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-92, 180);
								tft.println("V1");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-92, 180);
								tft.println("V1");
						}

							if ((digitalRead(_VQS_V2))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-52, 180);
								tft.println("V2");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-52, 180);
								tft.println("V2");
						}

							if ((digitalRead(_VQS_V3))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-12, 180);
								tft.println("V3");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-12, 180);
								tft.println("V3");
						}

							if ((digitalRead(_VQS_V4))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+28, 180);
								tft.println("V4");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+28, 180);
								tft.println("V4");
						}

							if ((digitalRead(_VQS_V5))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+68, 180);
								tft.println("V5");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+68, 180);
								tft.println("V5");
						}

							if ((digitalRead(_VQS_V6))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-92, 220);
								tft.println("V6");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-92, 220);
								tft.println("V6");
						}

							if ((digitalRead(_VQS_V7))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-52, 220);
								tft.println("V7");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-52, 220);
								tft.println("V7");
						}

							if ((digitalRead(_VQS_V8))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-12, 220);
								tft.println("V8");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-12, 220);
								tft.println("V8");
						}

							if ((digitalRead(_VQS_V9))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+28, 220);
								tft.println("V9");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+28, 220);
								tft.println("V9");
						}

							if ((digitalRead(_VQS_V10))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+62, 220);
								tft.println("V10");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+62, 220);
								tft.println("V10");
						}

							if ((digitalRead(_DIP1))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-92, 260);
								tft.println("D1");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-92, 260);
								tft.println("D1");
						}

							if ((digitalRead(_DIP2))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-52, 260);
								tft.println("D2");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-52, 260);
								tft.println("D2");
						}

							if ((digitalRead(_DIP3))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line-12, 260);
								tft.println("D3");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line-12, 260);
								tft.println("D3");
						}

							if ((digitalRead(_DIP4))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+28, 260);
								tft.println("D4");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+28, 260);
								tft.println("D4");
						}

							if ((digitalRead(_DIP5))==LOW)
						    {
								tft.setTextColor(GREEN);
								tft.setCursor(c_line+68, 260);
								tft.println("D5");
							}
							else
						    {
								tft.setTextColor(BLUE);
								tft.setCursor(c_line+68, 260);
								tft.println("D5");
						}






						if ((digitalRead(_S_Back))==LOW)
					    {
							Tool_Active = 0;
							menu.moveUp();
							Menu_Change = 1;
							menu.moveDown();

					    	// Wait for switch to be released
					    	while ((digitalRead(_S_Back)) == LOW){}
					    }
					}
				break;
		}
		case 3:{

			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("NRF24L01 Data");

			delay(1000);

			tft.setTextColor(BLACK);

			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("NRF24L01 Data");

			tft.setTextColor(GREEN);

			tft.setCursor(5, 5);
			tft.println("Attached");
			tft.setCursor(5, 25);
			tft.println("Variant");


			tft.setCursor(130, 5);
			if(Radio_Attached == 0){tft.println("No");}
			else if(Radio_Attached == 1){tft.println("Yes");}

			uint8_t Variant = radio.isPVariant();
			tft.setCursor(130, 25);
			if(Variant == 0){tft.println("nRF24L01");}
			else if(Variant == 1){tft.println("nRF24L01+");}
			else{tft.println("Fail");}

			tft.setCursor(5, 65);
			tft.println("Data Rate");
			tft.setCursor(5, 85);
			tft.println("Setting to");
			tft.setCursor(5, 105);
			tft.println("Response");
			tft.setCursor(5, 125);
			tft.println("Result");

			  for (int testnum = 0; testnum < 3; testnum++) {


				if(testnum == 0){
					radio.setDataRate(RF24_1MBPS);
					tft.setCursor(130, 65);
					tft.println("Test 1/3");
					tft.setCursor(130, 85);
					tft.println("1 Mbps");
				}
				else if(testnum == 1){
					radio.setDataRate(RF24_2MBPS);
					tft.setCursor(130, 65);
					tft.println("Test 2/3");
					tft.setCursor(130, 85);
					tft.println("2 Mbps");
				}
				else if(testnum == 2){
					radio.setDataRate(RF24_250KBPS);
					tft.setCursor(130, 65);
					tft.println("Test 3/3");
					tft.setCursor(130, 85);
					tft.println("250 kbps");
				}

				uint8_t DataRate = radio.getDataRate();
				tft.setCursor(130, 105);
				if(DataRate == 0){tft.println("1 Mbps");}
				else if(DataRate == 1){tft.println("2 Mbps");}
				else if(DataRate == 2){tft.println("250 kbps");}

				tft.setCursor(130, 125);
				if(testnum == DataRate){tft.println("Match");}else{tft.println("Fail");}

				delay(1000);

				tft.fillRect(130,65,240,145,BLACK);
			}

			radio.setDataRate(RF24_1MBPS);



			tft.setCursor(5, 165);
			tft.println("Power Amp");
			tft.setCursor(5, 185);
			tft.println("Setting to");
			tft.setCursor(5, 205);
			tft.println("Response");
			tft.setCursor(5, 225);
			tft.println("Result");

			  for (int testnum = 0; testnum < 4; testnum++) {


				if(testnum == 0){
					radio.setPALevel(RF24_PA_MIN);
					tft.setCursor(130, 165);
					tft.println("Test 1/4");
					tft.setCursor(130, 185);
					tft.println("Min");
				}
				else if(testnum == 1){
					radio.setPALevel(RF24_PA_LOW);
					tft.setCursor(130, 165);
					tft.println("Test 2/4");
					tft.setCursor(130, 185);
					tft.println("Low");
				}
				else if(testnum == 2){
					radio.setPALevel(RF24_PA_HIGH);
					tft.setCursor(130, 165);
					tft.println("Test 3/4");
					tft.setCursor(130, 185);
					tft.println("High");
				}
				else if(testnum == 3){
					radio.setPALevel(RF24_PA_MAX);
					tft.setCursor(130, 165);
					tft.println("Test 4/4");
					tft.setCursor(130, 185);
					tft.println("Max");
				}

				uint8_t PALevel = radio.getPALevel();
				tft.setCursor(130, 205);
				if(PALevel == 0){tft.println("Min");}
				else if(PALevel == 1){tft.println("Low");}
				else if(PALevel == 2){tft.println("High");}
				else if(PALevel == 3){tft.println("Max");}

				tft.setCursor(130, 225);
				if(testnum == PALevel){tft.println("Match");}else{tft.println("Fail");}

				delay(1000);

				tft.fillRect(130,165,240,245,BLACK);
			}

			radio.setPALevel(RF24_PA_LOW);

			while(Tool_Active==3){



				if ((digitalRead(_S_Back))==LOW)
			    {
					Tool_Active = 0;
					menu.moveUp();
					Menu_Change = 1;
					menu.moveDown();

			    	// Wait for switch to be released
			    	while ((digitalRead(_S_Back)) == LOW){}
			    }
			}
		break;
		}
		case 4:{

			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("XBee Data");

			delay(1000);

			tft.setTextColor(BLACK);

			tft.setCursor(5, 5);
			tft.println("This tool displays");
			tft.setCursor(5, 25);
			tft.println("XBee Data");

/*
			Serial.print("+++");

char inputString[20] = "";         // a string to hold incoming data

int timeout = 100;
while(timeout>0){
 if (Serial.available() > 0) {

    byte inByte = Serial.read();

    delay(10);

}
timeout--;
  }*/

			tft.setTextColor(GREEN);
			tft.setTextSize(2);
			tft.setCursor(5, 5);
			tft.println("You need to send");
			tft.setCursor(5, 25);
			tft.println("+++ to the XBee");
			tft.setCursor(5, 45);
			tft.println("then you should get");
			tft.setCursor(5, 65);
			tft.println("OK back. After that");
			tft.setCursor(5, 85);
			tft.println("you send the AT command");
			tft.setCursor(5, 105);
			tft.println("you are trying to use.");
			tft.setCursor(5, 125);
			tft.println("I haven't figured it");
			tft.setCursor(5, 145);
			tft.println("out yet.");

					while(Tool_Active==4){



						if ((digitalRead(_S_Back))==LOW)
					    {
							Tool_Active = 0;
							menu.moveUp();
							Menu_Change = 1;
							menu.moveDown();

					    	// Wait for switch to be released
					    	while ((digitalRead(_S_Back)) == LOW){}
					    }
					}
				break;
		}
			case 5:{

	tft.setCursor(5, 5);
    tft.setTextColor(YELLOW); tft.setTextSize(2);
    tft.println("Screen ID:");
	tft.setCursor(110, 5);
    tft.println(identifier);
	delay(2000);

// Fillscreen Test
	tft.fillRect(0,0,240,320,BLACK);
	tft.fillRect(0,0,240,320,RED);
	tft.fillRect(0,0,240,320,GREEN);
	tft.fillRect(0,0,240,320,BLUE);
	tft.fillRect(0,0,240,320,BLACK);

// Test Test
    tft.setCursor(0, 0);
    tft.setTextColor(WHITE);  tft.setTextSize(1);
    tft.println("Hello World!");
    tft.setTextColor(YELLOW); tft.setTextSize(2);
    tft.println(123.45);
    tft.setTextColor(RED);    tft.setTextSize(3);
    tft.println(0xDEADBEEF, HEX);
    tft.println();
    tft.setTextColor(GREEN);
    tft.setTextSize(5);
    tft.println("Groop");
    tft.setTextSize(2);
    tft.println("I implore thee,");
    tft.setTextSize(1);
    tft.println("my foonting turlingdromes.");
    tft.println("And hooptiously drangle me");
    tft.println("with crinkly bindlewurdles,");
    tft.println("Or I will rend thee");
    tft.println("in the gobberwarts");
    tft.println("with my blurglecruncheon,");
    tft.println("see if I don't!");

	delay(2000);

// Test Lines
    int           x1, y1, x2, y2,
                  w = tft.width(),
                  h = tft.height();

	tft.fillRect(0,0,240,320,BLACK);

    x1 = y1 = 0;
    y2    = h - 1;
    for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);
    x2    = w - 1;
    for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);

	tft.fillRect(0,0,240,320,BLACK);

    x1    = w - 1;
    y1    = 0;
    y2    = h - 1;
    for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);
    x2    = 0;
    for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);

	tft.fillRect(0,0,240,320,BLACK);

    x1    = 0;
    y1    = h - 1;
    y2    = 0;
    for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);
    x2    = w - 1;
    for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);

	tft.fillRect(0,0,240,320,BLACK);

    x1    = w - 1;
    y1    = h - 1;
    y2    = 0;
    for (x2 = 0; x2 < w; x2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);
    x2    = 0;
    for (y2 = 0; y2 < h; y2 += 6) tft.drawLine(x1, y1, x2, y2, BLUE);


// Filled Circle Test
int radius = 10;
int x, y, r2 = radius * 2;

    tft.fillScreen(BLACK);
    start = micros();
    for (x = radius; x < w; x += r2) {
        for (y = radius; y < h; y += r2) {
            tft.fillCircle(x, y, radius, GREEN);
        }
    }

// Circle Test
                        w = tft.width()  + radius;
                        h = tft.height() + radius;

    // Screen is not cleared for this one -- this is
    // intentional and does not affect the reported time.
    for (x = 0; x < w; x += r2) {
        for (y = 0; y < h; y += r2) {
            tft.drawCircle(x, y, radius, MAGENTA);
        }
    }

// Triangle Test
    int           n, i, cx = tft.width()  / 2 - 1,
                        cy = tft.height() / 2 - 1;

    tft.fillRect(0,0,240,320,BLACK);
    n     = min(cx, cy);
    for (i = 0; i < n; i += 5) {
        tft.drawTriangle(
            cx    , cy - i, // peak
            cx - i, cy + i, // bottom left
            cx + i, cy + i, // bottom right
            tft.color565(0, 0, i));
    }

// Filled Triangle Test

	tft.fillRect(0,0,240,320,BLACK);
    for (i = min(cx, cy); i > 10; i -= 5) {
        tft.fillTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i,
                         tft.color565(0, i, i));
        tft.drawTriangle(cx, cy - i, cx - i, cy + i, cx + i, cy + i,
                         tft.color565(i, i, 0));
    }

// Round Rectangle Test
    int           i2, red, step;

	tft.fillRect(0,0,240,320,BLACK);
    w     = min(tft.width(), tft.height());
    red = 0;
    step = (256 * 6) / w;
    for (i = 0; i < w; i += 6) {
        i2 = i / 2;
        red += step;
        tft.drawRoundRect(cx - i2, cy - i2, i, i, i / 8, tft.color565(red, 0, 0));
    }

// Rectangle Test
    int           green;

	tft.fillRect(0,0,240,320,BLACK);
    green = 256;
    step = (256 * 6) / min(tft.width(), tft.height());
    for (i = min(tft.width(), tft.height()); i > 20; i -= 6) {
        i2 = i / 2;
        green -= step;
        tft.fillRoundRect(cx - i2, cy - i2, i, i, i / 8, tft.color565(0, green, 0));
    }

	tft.fillRect(0,0,240,320,BLACK);

	tft.setTextColor(GREEN);
					tft.setTextSize(2);
					tft.setCursor(5, 5);
				tft.println("Test Finished");

	while(Tool_Active==5){



							if ((digitalRead(_S_Back))==LOW)
						    {
								Tool_Active = 0;
								menu.moveUp();
								Menu_Change = 1;
								menu.moveDown();

						    	// Wait for switch to be released
						    	while ((digitalRead(_S_Back)) == LOW){}
						    }
						}
					break;
		}
			case 6:{

// most mcufriend shields use these pins and Portrait mode:
uint8_t YP = A1;  // must be an analog pin, use "An" notation!
uint8_t XM = A2;  // must be an analog pin, use "An" notation!
uint8_t YM = 29;   // can be a digital pin
uint8_t XP = 27;   // can be a digital pin
uint8_t SwapXY = 0;

uint16_t TS_LEFT = 920;
uint16_t TS_RT  = 150;
uint16_t TS_TOP = 940;
uint16_t TS_BOT = 120;
char *name = "Unknown controller";

// For better pressure precision, we need to know the resistance
// between X+ and X- Use any multimeter to read it
// For the one we're using, its 300 ohms across the X plate
TouchScreen ts = TouchScreen(XP, YP, XM, YM, 300);
TSPoint tp;

#define MINPRESSURE 20
#define MAXPRESSURE 1000

#define SWAP(a, b) {uint16_t tmp = a; a = b; b = tmp;}

int16_t BOXSIZE;
int16_t PENRADIUS = 3;
uint16_t identifier, oldcolor, currentcolor;
uint8_t Orientation = 2;    //PORTRAIT

    ts = TouchScreen(XP, YP, XM, YM, 300);     //call the constructor AGAIN with new values.

    tft.setRotation(Orientation);
    tft.fillScreen(BLACK);

tft.setCursor(0, 0);
    tft.setTextSize(2);
    tft.print(F("Found "));
    tft.print(name);
    tft.println(F(" LCD"));
    tft.setTextSize(1);
    tft.print(F("ID=0x"));
    tft.println(identifier, HEX);
    tft.println("Screen is " + String(tft.width()) + "x" + String(tft.height()));
    tft.println("Calibration is: ");
    tft.println("LEFT = " + String(TS_LEFT) + " RT  = " + String(TS_RT));
    tft.println("TOP  = " + String(TS_TOP)  + " BOT = " + String(TS_BOT));
    tft.print("\nWiring is: ");
    if (SwapXY) {
        tft.setTextColor(CYAN);
        tft.setTextSize(2);
    }
    tft.println(SwapXY ? "SWAPXY" : "PORTRAIT");
    tft.println("YP=" + String(YP)  + " XM=" + String(XM));
    tft.println("YM=" + String(YM)  + " XP=" + String(XP));
    tft.setTextSize(2);
    tft.setTextColor(RED);
    tft.setCursor((tft.width() - 48) / 2, (tft.height() * 2) / 4);
    tft.print("EXIT");
    tft.setTextColor(YELLOW, BLACK);
    tft.setCursor(0, (tft.height() * 6) / 8);
    tft.print("Touch screen for loc");
    while (1) {
        tp = ts.getPoint();
        pinMode(XM, OUTPUT);
        pinMode(YP, OUTPUT);
        pinMode(XP, OUTPUT);
        pinMode(YM, OUTPUT);
        if (tp.z < MINPRESSURE || tp.z > MAXPRESSURE) continue;
        if (tp.x > 450 && tp.x < 570  && tp.y > 450 && tp.y < 570) break;
        tft.setCursor(0, (tft.height() * 3) / 4);
        tft.print("tp.x=" + String(tp.x) + " tp.y=" + String(tp.y) + "   ");
    }

    BOXSIZE = tft.width() / 6;
    tft.fillScreen(BLACK);

    tft.fillRect(0, 0, BOXSIZE, BOXSIZE, RED);
    tft.fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, YELLOW);
    tft.fillRect(BOXSIZE * 2, 0, BOXSIZE, BOXSIZE, GREEN);
    tft.fillRect(BOXSIZE * 3, 0, BOXSIZE, BOXSIZE, CYAN);
    tft.fillRect(BOXSIZE * 4, 0, BOXSIZE, BOXSIZE, BLUE);
    tft.fillRect(BOXSIZE * 5, 0, BOXSIZE, BOXSIZE, MAGENTA);

    tft.drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
    currentcolor = RED;
    delay(1000);

						while(Tool_Active==6){

uint16_t xpos, ypos;  //screen coordinates
    tp = ts.getPoint();   //tp.x, tp.y are ADC values

    // if sharing pins, you'll need to fix the directions of the touchscreen pins
    pinMode(XM, OUTPUT);
    pinMode(YP, OUTPUT);
    pinMode(XP, OUTPUT);
    pinMode(YM, OUTPUT);
    //    digitalWrite(XM, HIGH);
    //    digitalWrite(YP, HIGH);
    // we have some minimum pressure we consider 'valid'
    // pressure of 0 means no pressing!

    if (tp.z > MINPRESSURE && tp.z < MAXPRESSURE) {
        // is controller wired for Landscape ? or are we oriented in Landscape?
        if (SwapXY != (Orientation & 1)) SWAP(tp.x, tp.y);
        // scale from 0->1023 to tft.width  i.e. left = 0, rt = width
        // most mcufriend have touch (with icons) that extends below the TFT
        // screens without icons need to reserve a space for "erase"
        // scale the ADC values from ts.getPoint() to screen values e.g. 0-239
        xpos = map(tp.x, TS_LEFT, TS_RT, 0, tft.width());
        ypos = map(tp.y, TS_TOP, TS_BOT, 0, tft.height());

        // are we in top color box area ?
        if (ypos < BOXSIZE) {               //draw white border on selected color box
            oldcolor = currentcolor;

            if (xpos < BOXSIZE) {
                currentcolor = RED;
                tft.drawRect(0, 0, BOXSIZE, BOXSIZE, WHITE);
            } else if (xpos < BOXSIZE * 2) {
                currentcolor = YELLOW;
                tft.drawRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, WHITE);
            } else if (xpos < BOXSIZE * 3) {
                currentcolor = GREEN;
                tft.drawRect(BOXSIZE * 2, 0, BOXSIZE, BOXSIZE, WHITE);
            } else if (xpos < BOXSIZE * 4) {
                currentcolor = CYAN;
                tft.drawRect(BOXSIZE * 3, 0, BOXSIZE, BOXSIZE, WHITE);
            } else if (xpos < BOXSIZE * 5) {
                currentcolor = BLUE;
                tft.drawRect(BOXSIZE * 4, 0, BOXSIZE, BOXSIZE, WHITE);
            } else if (xpos < BOXSIZE * 6) {
                currentcolor = MAGENTA;
                tft.drawRect(BOXSIZE * 5, 0, BOXSIZE, BOXSIZE, WHITE);
            }

            if (oldcolor != currentcolor) { //rub out the previous white border
                if (oldcolor == RED) tft.fillRect(0, 0, BOXSIZE, BOXSIZE, RED);
                if (oldcolor == YELLOW) tft.fillRect(BOXSIZE, 0, BOXSIZE, BOXSIZE, YELLOW);
                if (oldcolor == GREEN) tft.fillRect(BOXSIZE * 2, 0, BOXSIZE, BOXSIZE, GREEN);
                if (oldcolor == CYAN) tft.fillRect(BOXSIZE * 3, 0, BOXSIZE, BOXSIZE, CYAN);
                if (oldcolor == BLUE) tft.fillRect(BOXSIZE * 4, 0, BOXSIZE, BOXSIZE, BLUE);
                if (oldcolor == MAGENTA) tft.fillRect(BOXSIZE * 5, 0, BOXSIZE, BOXSIZE, MAGENTA);
            }
        }
        // are we in drawing area ?
        if (((ypos - PENRADIUS) > BOXSIZE) && ((ypos + PENRADIUS) < tft.height())) {
            tft.fillCircle(xpos, ypos, PENRADIUS, currentcolor);
        }
        // are we in erase area ?
        if (ypos > tft.height() - 10) {
            // press the bottom of the screen to erase
            tft.fillRect(0, BOXSIZE, tft.width(), tft.height() - BOXSIZE, BLACK);
        }
    }

							if ((digitalRead(_S_Back))==LOW)
						    {
								Tool_Active = 0;
								menu.moveUp();
								Menu_Change = 1;
								menu.moveDown();

						    	// Wait for switch to be released
						    	while ((digitalRead(_S_Back)) == LOW){}
						    }
						}
					break;
		}
	}
}


